@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -67,6 +67,7 @@ int Xorriso_cdrskin_help(struct XorrisO *xorriso, int flag)
 "\t-toc\t\tretrieve and print TOC/PMA data",
 "\t-atip\t\tretrieve media state, print \"Is *erasable\"",
 "\t-multi\t\tgenerate a TOC that allows multi session",
+"\t--multi_if_possible\tapply -multi if the medium supports it",
 "\t-waiti\t\twait until input is available before opening SCSI",
 "\t-tao\t\tWrite disk in TAO mode.",
 "\t-dao\t\tWrite disk in SAO mode.",
@@ -77,11 +78,14 @@ int Xorriso_cdrskin_help(struct XorrisO *xorriso, int flag)
 "\t-isosize\tUse iso9660 file system size for next data track",
 "\t-pad\t\tpadsize=30k",
 "\t-nopad\t\tDo not pad",
+"\tminbuf=\t\tKeep drive buffer hungry",
+"\tmodesty_on_drive=\tLike minbuf=, but with more parameters",
+"\t-immed\t\tEquivalent to minbuf=75",
 "\t--grow_overwriteable_iso\temulate multi-session on DVD+RW, BD-RE",
 "\twrite_start_address=#\t\twrite to byte address on DVD+RW, BD-RE",
 "\tstream_recording=on|number\ttry to get full speed on DVD-RAM, BD",
 "\tdvd_obs=default|32k|64k\t\tbytes per DVD/BD write operation",
-"\tstdio_sync=on|off|number\twhether to fsync output to \"stdio:\"",
+"\tstdio_sync=on|off|end|number\twhether to fsync output to \"stdio:\"",
 "\t--no_rc\t\tDo not execute xorriso startup files",
 "\t-help\t\tprint this text to stderr and exit emulation",
 "Actually this is the integrated ISO RockRidge filesystem manipulator xorriso",
@@ -108,7 +112,8 @@ int Xorriso_cdrskin(struct XorrisO *xorriso, char *whom, int argc, char **argv,
  int do_atip= 0, do_checkdrive= 0, do_eject= 0, do_scanbus= 0;
  int do_toc= 0, do_verbous= 0, do_version= 0, do_help= 0, do_waiti= 0;
  int do_multi= 0, do_msinfo= 0, do_grow= 0, do_isosize= 0, do_xa1= 0;
- double write_start_address= -1.0, tsize= -1.0;
+ int do_auto_close= 0;
+ double write_start_address= -1.0, tsize= -1.0, mem_auto_close;
  char *track_source= NULL, *dev_adr= NULL, *cpt;
  char mem_report_about_text[80], *report_about= "SORRY", blank_mode[80];
  char speed[80], *argpt;
@@ -118,10 +123,10 @@ int Xorriso_cdrskin(struct XorrisO *xorriso, char *whom, int argc, char **argv,
    "timeout=", "debug=", "kdebug=", "kd=", "driver=", "ts=",
    "pregap=", "defpregap=", "mcn=", "isrc=", "index=", "textfile=",
    "pktsize=", "cuefile=",
-   "gracetime=", "minbuf=",
+   "gracetime=",
 
    "assert_write_lba=", "fifo_start_at=", "dev_translation=",
-   "drive_scsi_dev_family=", "fallback_program=", "modesty_on_drive=",
+   "drive_scsi_dev_family=", "fallback_program=",
    "tao_to_sao_tsize=", 
    
    "direct_write_amount=", "msifile=",
@@ -165,6 +170,7 @@ static char blank_help[][80]= {
 };
 
  mem_do_close= xorriso->do_close;
+ mem_auto_close= xorriso->auto_close;
  Xorriso_alloc_meM(track_source, char, SfileadrL);
  Xorriso_alloc_meM(dev_adr, char, SfileadrL);
 
@@ -297,6 +303,10 @@ no_volunteer:;
      do_msinfo= 1;
    } else if(strcmp(argpt, "-multi")==0) {
      do_multi= 1;
+     do_auto_close= 0;
+   } else if(strcmp(argv[i], "--multi_if_possible") == 0) {
+     do_multi= 1;
+     do_auto_close= 1;
    } else if(strcmp(argpt, "-nopad")==0) {
      xorriso->padding= 0;
    } else if(strcmp(argv[i], "--no_rc")==0) { /* intentional: argv[i] */
@@ -325,6 +335,17 @@ no_volunteer:;
              strncmp(argpt, "stream_recording=", 17)==0) {
      cpt= strchr(argpt, '=')+1;
      Xorriso_option_stream_recording(xorriso, cpt, 0);
+
+   } else if(strcmp(argpt, "-immed") == 0) {
+     Xorriso_option_modesty_on_drive(xorriso, "75", 0);
+
+   } else if(strncmp(argpt, "-minbuf=", 8) == 0 ||
+             strncmp(argpt, "minbuf=", 7) == 0 ||
+             strncmp(argpt, "-modesty_on_drive=", 18) == 0 ||
+             strncmp(argpt, "modesty_on_drive=", 17) == 0) {
+     cpt= strchr(argpt, '=') + 1;
+     Xorriso_option_modesty_on_drive(xorriso, cpt, 0);
+
    } else if(strncmp(argpt, "-stdio_sync=", 12)==0 ||
              strncmp(argpt, "stdio_sync=", 11)==0) {
      cpt= strchr(argpt, '=') + 1;
@@ -510,9 +531,15 @@ no_volunteer:;
  }
  if(track_source[0]) {
    xorriso->do_close= !do_multi;
+   xorriso->auto_close= do_auto_close;
    ret= Xorriso_burn_track(xorriso, (off_t) write_start_address,
                            track_source, (off_t) tsize,
                    (!!do_grow) | ((!!do_isosize) << 1) | ((do_xa1 == 1) << 2));
+   if(ret == 2) {
+     ret= Xorriso_retry_burn_track(xorriso, (off_t) write_start_address,
+                           track_source, (off_t) tsize,
+                   (!!do_grow) | ((!!do_isosize) << 1) | ((do_xa1 == 1) << 2));
+   }
    aq_ret= Xorriso_reaquire_outdev(xorriso, 2*(ret>0));
    if(ret<=0 && ret<aq_ret)
      goto ex;
@@ -534,6 +561,7 @@ ex:;
  }
  Xorriso_option_report_about(xorriso, mem_report_about_text, 0);
  xorriso->do_close= mem_do_close;
+ xorriso->auto_close= mem_auto_close;
  Xorriso_free_meM(dev_adr);
  Xorriso_free_meM(track_source);
  return(ret);
@@ -583,7 +611,8 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "--old-root-no-md5", "--old-root-devno", "--old-root-no-ino",
     "--no_rc", "--norock", "-hfsplus", "-fat", "-chrp-boot-part",
     "-isohybrid-gpt-basdat", "-isohybrid-gpt-hfsplus",
-    "-isohybrid-apm-hfsplus", "--grub2-boot-info",
+    "-isohybrid-apm-hfsplus", "--grub2-boot-info", "-joliet-utf16",
+    "-appended_part_as_gpt",
     ""
  };
  static char arg1_options[][41]= {
@@ -611,6 +640,7 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "-volid", "-old-exclude",
     "-alpha-boot", "-hppa-cmdline", "-hppa-kernel-32", "-hppa-kernel-64",
     "-hppa-bootloader", "-hppa-ramdisk", "-mips-boot", "-mipsel-boot",
+    "-alpha-boot",
     "-jigdo-jigdo", "-jigdo-template", "-jigdo-min-file-size",
     "-jigdo-force-md5", "-jigdo-exclude", "-jigdo-map", "-md5-list",
     "-jigdo-template-compress",
@@ -621,6 +651,7 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "-rr_reloc_dir", "-hfsplus-serial-no", "-prep-boot-part", "-efi-boot-part",
     "-hfsplus-block-size", "-apm-block-size", "--grub2-mbr",
     "--grub2-sparc-core", "--sort-weight-list", "--sort-weight-patterns",
+    "-hppa-hdrversion","-file_name_limit",
     ""
  };
  static char arg2_options[][41]= {
@@ -747,6 +778,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -disallow_dir_id_ext        Do not allow dot in ISO directory names",
 "  -J, -joliet                 Generate Joliet directory information",
 "  -joliet-long                Allow Joliet file names to be 103 Unicode characters",
+"  -joliet-utf16               Encode Joliet names in UTF-16BE rather than UCS-2",
 "  -U, -untranslated-filenames Allow Untranslated filenames (for HPUX & AIX - violates ISO9660).",
 "  -untranslated_name_len LEN  Allow up to LEN (1..96) name characters (heavily violates ISO9660).",
 "  -allow-lowercase            Allow lower case characters in addition to the current character set (violates ISO9660)",
@@ -779,6 +811,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -R, -rock                   Generate Rock Ridge directory information",
 "  -r, -rational-rock          Generate rationalized Rock Ridge directory information",
 "  --norock                    Disable Rock Ridge. (Strongly discouraged !)",
+"  -file_name_limit LEN        Set truncation limit for Rock Ridge names",
 "  --hardlinks                 Record eventual hard link relations of files",
 "  --acl                       Record eventual ACLs of files",
 "  --xattr                     Record eventual user space xattr of files",
@@ -838,13 +871,22 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -mipsel-boot FILE           Set mipsel boot image name (relative to image root)",
 "  -B FILES, -sparc-boot FILES Set sparc boot image names",
 "  -sparc-label label text     Set sparc boot disk label",
+"  -hppa-cmdline CMDLINE       Set hppa boot command line",
+"  -hppa-kernel-32 FILE        Set hppa 32-bit image name (relative to image root)",
+"  -hppa-kernel-64 FILE        Set hppa 64-bit image name (relative to image root)",
+"  -hppa-bootloader FILE       Set hppa boot loader file name (relative to image root)",
+"  -hppa-ramdisk FILE          Set hppa ramdisk file name (relative to image root)",
+"  -hppa-hdrversion NUMBER     Set hppa PALO header version to 4 or 5",
+"  -alpha-boot FILE            Set alpha boot image name (relative to image root)",
 "  --grub2-sparc-core FILE     Set path of core file for disk label patching",
 "  -efi-boot-part DISKFILE|--efi-boot-image",
 "                              Set data source for EFI System Partition",
-"  -chrp-boot-part             Mark ISO image size by MBR partition type 0x41",
-"  -prep-boot-part DISKFILE    Set data source for MBR partition type 0x96",
+"  -chrp-boot-part             Mark ISO image size by MBR partition type 0x96",
+"  -chrp-boot                  Alias of -chrp-boot-part",
+"  -prep-boot-part DISKFILE    Set data source for MBR partition type 0x41",
 "  -append_partition NUMBER TYPE FILE",
 "                              Append FILE after image. TYPE is hex: 0x..",
+"  -appended_part_as_gpt       mark appended partitions in GPT instead of MBR.",
 "  --modification-date=YYYYMMDDhhmmsscc",
 "                              Override date of creation and modification",
 "  -isohybrid-mbr FILE         Set SYSLINUX mbr/isohdp[fp]x*.bin for isohybrid",
@@ -939,8 +981,6 @@ int Xorriso_genisofs_hide_list(struct XorrisO *xorriso, char *whom,
    return(0);
  }
  ret= Xorriso_afile_fopen(xorriso, adr, "rb", &fp, 0);
- if(ret <= 0)
-   return(0);
  if(ret <= 0)
    return(0);
  while(1) {
@@ -1128,6 +1168,10 @@ int Xorriso_genisofs_path_pecul(struct XorrisO *xorriso, int *was_path,
  /* mkisofs records mtime in ECMA-119 and Joliet
  */
  Xorriso_relax_compliance(xorriso, "rec_mtime", 0);
+ /* mkisofs is substantially faster than xorriso by not fsyncing
+ */
+ if(xorriso->stdio_sync_is_default)
+   Xorriso_option_stdio_sync(xorriso, "off", 0);
 
  Xorriso_free_meM(sfe);
  ret= 1;
@@ -1141,7 +1185,7 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
                      int argc, char **argv, int flag)
 {
  int ret, i, j, was_path= 0, was_other_option= 0, mem_graft_points, mem;
- int do_print_size= 0, fd, idx, iso_level= -1;
+ int do_print_size= 0, fd, idx, iso_level= -1, palohdrversion;
  int was_failure= 0, fret, lower_r= 0, zero= 0;
  int dir_mode= -1, file_mode= -1, count, partition_number;
  int allow_dir_id_ext= -1;
@@ -1156,7 +1200,7 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
  char *old_root= NULL, *argpt, *hargv[1];
  char *boot_path, partno_text[8], *iso_rr_pt, *disk_pt, *rpt, *wpt;
  char *rm_merge_args[3], *rr_reloc_dir_pt= NULL;
- char *sort_weight_args[4], *bless_args[6];
+ char *sort_weight_args[4], *bless_args[6], *sa_path;
 
  struct stat stbuf;
 
@@ -1415,8 +1459,14 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
      xorriso->no_emul_toc|= 1;
    } else if(strcmp(argpt, "-log-file") == 0) {
      /* already handled before this loop */;
+     i++;
    } else if(strcmp(argpt, "-gui") == 0) {
      xorriso->pacifier_interval= 0.25;
+   } else if(strcmp(argpt, "-file_name_limit") == 0) {
+     if(i+1>=argc)
+       goto not_enough_args;
+     i++;
+     Xorriso_option_file_name_limit(xorriso, argv[i], 0);
    } else {
      if(argv[i][0] == '-') {
        ret= Xorriso_genisofs_fused_options(xorriso, whom, argv[i] + 1,
@@ -1578,6 +1628,8 @@ illegal_c:;
    } else if(strcmp(argpt, "-joliet-long")==0) {
      Xorriso_relax_compliance(xorriso,
                               "joliet_long_paths:joliet_long_names", 0);
+   } else if(strcmp(argpt, "-joliet-utf16")==0) {
+     Xorriso_relax_compliance(xorriso, "joliet_utf16", 0);
    } else if(strcmp(argpt, "-fat") == 0) {
      /* was already handled in first argument scan */;
    } else if(strcmp(argpt, "-hfs-bless") == 0 ||
@@ -1765,7 +1817,9 @@ not_enough_args:;
              strcmp(argpt, "-partition_sec_hd") == 0 ||
              strcmp(argpt, "-partition_cyl_align") == 0 ||
              strcmp(argpt, "-isohybrid-mbr") == 0 ||
-             strcmp(argpt, "--grub2-mbr") == 0) {
+             strcmp(argpt, "--grub2-mbr") == 0 ||
+             strncmp(argpt, "-hppa-", 6) == 0 ||
+             strcmp(argpt, "-alpha-boot") == 0) {
      if(i+1>=argc)
        goto not_enough_args;
      delay_opt_list[delay_opt_count++]= i;
@@ -1901,7 +1955,8 @@ not_enough_args:;
      if(ret <= 0)
        goto problem_handler_2;
 
-   } else if(strcmp(argpt, "-chrp-boot-part") == 0) {
+   } else if(strcmp(argpt, "-chrp-boot-part") == 0 ||
+             strcmp(argpt, "-chrp-boot") == 0) {
      ret= Xorriso_option_boot_image(xorriso, "any", "chrp_boot_part=on", 0);
      if(ret <= 0)
        goto problem_handler_2;
@@ -1931,6 +1986,9 @@ not_enough_args:;
      if(ret <= 0)
        goto problem_handler_2;
 
+   } else if(strcmp(argpt, "-appended_part_as_gpt") == 0) {
+     xorriso->appended_as_gpt= 1;
+
    } else if(strcmp(argpt, "-B") == 0 ||
              strcmp(argpt, "-sparc-boot") == 0) {
      i++;
@@ -1983,8 +2041,10 @@ not_enough_args:;
      if(i+1>=argc)
        goto not_enough_args;
      i++;
-     strncpy(xorriso->ascii_disc_label, argv[i], Xorriso_disc_label_sizE - 1);
-     xorriso->ascii_disc_label[Xorriso_disc_label_sizE - 1] = 0;
+     sprintf(sfe, "sparc_label=%s", argv[i]);
+     ret= Xorriso_option_boot_image(xorriso, "any", sfe, 0);
+     if(ret <= 0)
+       goto problem_handler_2;
 
    } else if(strcmp(argpt, "--grub2-sparc-core") == 0) {
      if(i+1>=argc)
@@ -2042,7 +2102,9 @@ rr_reloc_dir:;
        goto problem_handler_2;
      Xorriso_relax_compliance(xorriso, "deep_paths_off:long_paths_off", 0);
 
-   } else if(strcmp(argpt, "-log-file") == 0) {
+   } else if(strcmp(argpt, "-log-file") == 0 ||
+             strcmp(argpt, "-file_name_limit") == 0) {
+     i+= 1;
      /* was already handled before this loop */;
 
    } else if(strcmp(argpt, "-gui") == 0) {
@@ -2331,7 +2393,7 @@ problem_handler_2:;
      if(ret <= 0)
        goto problem_handler_boot;
    } else if(strcmp(argpt, "-isohybrid-gpt-basdat") == 0) {
-     xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x1fc) |
+     xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x0fc) |
                                      (1 << 2);
    } else if(strcmp(argpt, "-isohybrid-gpt-hfsplus") == 0) {
      xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x0fc) |
@@ -2350,7 +2412,11 @@ problem_handler_2:;
      if(i+1>=argc)
        goto not_enough_args;
      i++; 
-     ret= Xorriso_set_system_area_path(xorriso, argv[i], 0);
+     if(strcmp(argv[i], ".") == 0)
+       sa_path= "";
+     else
+       sa_path= argv[i];
+     ret= Xorriso_set_system_area_path(xorriso, sa_path, 0);
      if(ret <= 0)
        goto problem_handler_boot;
      if(strcmp(argpt, "-isohybrid-mbr")==0)
@@ -2392,6 +2458,31 @@ problem_handler_2:;
      if(ret <= 0)
        goto problem_handler_boot;
 
+   } else if(strncmp(argpt, "-hppa-", 6) == 0) {
+     if(i + 1 >= argc)
+       goto not_enough_args;
+     i++;
+     sprintf(sfe, "-as mkisofs %s %s", argpt, argv[i]);
+     palohdrversion= (xorriso->system_area_options >> 2) & 0x3f;
+     if(palohdrversion != 4)
+       palohdrversion= 5;
+     ret= Xorriso_coordinate_system_area(xorriso, palohdrversion, 0, sfe, 0);
+     if(ret <= 0)
+       goto ex;
+     ret= Xorriso_set_hppa_boot_parm(xorriso, argv[i], argpt + 6, 0);
+     if(ret <= 0)
+       goto problem_handler_boot;
+
+   } else if(strcmp(argpt, "-alpha-boot") == 0) {
+     if(i + 1 >= argc)
+       goto not_enough_args;
+     i++;
+     sprintf(sfe, "-as mkisofs %s %s", argpt, argv[i]);
+     ret= Xorriso_coordinate_system_area(xorriso, 6, 0, sfe, 0);
+     if(ret <= 0)
+       goto ex;
+     ret= Xorriso_set_alpha_boot(xorriso, argv[i], 0);
+
    } else if(strcmp(argpt, "-hfs-bless") == 0) {
      static char *bless_arg_data[6]= {
         "/", "-disk_path", "", "-exec", "set_hfs_bless", "p"};
@@ -2464,6 +2555,7 @@ ex:;
    free(weight_list);
  if(delay_opt_list != NULL)
    free(delay_opt_list);
+ Xorriso_free_meM(sort_file);
  Xorriso_free_meM(sfe);
  Xorriso_free_meM(adr);
  Xorriso_free_meM(pathspec);
