@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -42,8 +42,8 @@
 /* @param eff_path  returns resulting effective path.
                     Must provide at least SfileadrL bytes of storage.
    @param flag bit0= do not produce problem events (unless faulty path format)
-               bit1= work purely literally, do not use libisofs
-               bit2= (with bit1) this is an address in the disk world
+               bit1= do not use libisofs (work literally or in disk world)
+               bit2= (implies bit1) this is an address in the disk world
                bit3= return root directory as "/" and not as ""
                bit4= (with bit2) determine type of disk file eff_path
                      and return 0 if not existing
@@ -59,10 +59,13 @@ int Xorriso_normalize_img_path(struct XorrisO *xorriso, char *wd,
  IsoImage *volume;
  IsoDir *dir= NULL;
  IsoNode *node= NULL;
- char *path= NULL, *apt, *npt, *cpt;
+ char *path= NULL, *apt, *npt, *cpt, *link_target= NULL;
 
  Xorriso_alloc_meM(path, char, SfileadrL);
+ Xorriso_alloc_meM(link_target, char, SfileadrL);
 
+ if(flag & 4)
+   flag|= 2;
  if((flag&64) || !(flag&2)) {
    ret= Xorriso_get_volume(xorriso, &volume, 0);
    if(ret<=0)
@@ -129,7 +132,24 @@ bonked_root:;
          {ret= -1; goto ex;}
        }
        dir= iso_node_get_parent(node);
+
+       /* >>> Get absolute path of node as eff_path, to avoid link problems */; 
+
      }
+     if(flag & 4) {
+       /* Linux-ish local filesystem semantics:
+          If the parent is a symbolic link, do not ascend to its parent,
+          but to the parent of the link target.
+          >>> ??? Are there systems which do it differently ?
+       */
+       if(Sfile_type(eff_path, 0) == 3) {
+         ret= Xorriso_resolve_link(xorriso, eff_path, link_target, 2);
+         if(ret <= 0)
+           {ret= -1; goto ex;}
+         strcpy(eff_path, link_target);
+       }
+     }
+
      /* truncate eff_path */;
      cpt= strrchr(eff_path, '/');
      if(cpt==NULL) /* ??? if not flag&2 then this is a bug */
@@ -171,6 +191,7 @@ much_too_long:;
  ret= 1+!!is_dir;
 ex:;
  Xorriso_free_meM(path);
+ Xorriso_free_meM(link_target);
  return(ret);
 }
 
@@ -219,6 +240,7 @@ int Xorriso_node_get_dev(struct XorrisO *xorriso, IsoNode *node,
                bit3= stbuf is to be used without eventual ACL
                bit4= try to obtain a better st_nlink count if hardlinks
                      are enabled
+               bit5= do not look for truncated versions of path component
 */
 int Xorriso_fake_stbuf(struct XorrisO *xorriso, char *path, struct stat *stbuf,
                        IsoNode **node, int flag)
@@ -236,7 +258,8 @@ int Xorriso_fake_stbuf(struct XorrisO *xorriso, char *path, struct stat *stbuf,
    ret= Xorriso_get_volume(xorriso, &volume, 0);
    if(ret<=0)
      return(-1);
-   ret= Xorriso_node_from_path(xorriso, volume, path, node, !(flag&4));
+   ret= Xorriso_node_from_path(xorriso, volume, path, node,
+                               ((flag >> 4) & 2) | !(flag&4));
    if(ret<=0)
      *node= NULL;
  }
@@ -258,7 +281,8 @@ int Xorriso_fake_stbuf(struct XorrisO *xorriso, char *path, struct stat *stbuf,
    stbuf->st_mode|= S_IFLNK;
  else if(LIBISO_ISCHR(*node)) {
    stbuf->st_mode|= S_IFCHR;
-   Xorriso_node_get_dev(xorriso, *node, path, &(stbuf->st_rdev), 0);
+   Xorriso_node_get_dev(xorriso, *node, path, &dev_number, 0);
+   stbuf->st_rdev= dev_number;
  } else if(LIBISO_ISBLK(*node)) {
    stbuf->st_mode|= S_IFBLK;
    /* ts B11124:
@@ -705,6 +729,8 @@ int Xorriso_getfacl(struct XorrisO *xorriso, void *in_node, char *path,
 
  for(pass= 0; pass < 1 + (acl_text != NULL && !(flag & 2)); pass++) {
    if(pass) {
+     if(result_len == 0) 
+ break;
      *acl_text= calloc(result_len + 1, 1);
      if(*acl_text == NULL) {
        Xorriso_no_malloc_memory(xorriso, NULL, 0);
@@ -727,7 +753,8 @@ int Xorriso_getfacl(struct XorrisO *xorriso, void *in_node, char *path,
        if(pass) {
          sprintf(*acl_text + strlen(*acl_text), "%s\n", cpt);
        } else if(!(flag & 1)) {
-         Sfile_str(xorriso->result_line, cpt, 0);
+         if(Sfile_str(xorriso->result_line, cpt, 0) <= 0)
+           goto too_long;
          strcat(xorriso->result_line, "\n");
          Xorriso_result(xorriso, 0);
        }
@@ -749,8 +776,10 @@ int Xorriso_getfacl(struct XorrisO *xorriso, void *in_node, char *path,
          } else {
            xorriso->result_line[0]= 0;
            if(what == 0)
-             Sfile_str(xorriso->result_line, "default:", 0);
-           Sfile_str(xorriso->result_line, cpt, 1);
+             if(Sfile_str(xorriso->result_line, "default:", 0) <= 0)
+               goto too_long;
+           if(Sfile_str(xorriso->result_line, cpt, 1) <= 0)
+             goto too_long;
            result_len+= strlen(cpt) + 9;
          }
        } else
@@ -764,10 +793,20 @@ int Xorriso_getfacl(struct XorrisO *xorriso, void *in_node, char *path,
      }
    }
  }
- ret= 1;
+ if(result_len == 0) {
+   if(acl_text != NULL)
+     *acl_text= NULL;
+   ret= 2; 
+ } else {
+   ret= 1;
+ }
 ex:;
  iso_node_get_acl_text(node, &text, &d_text, 1 << 15);
  return(ret);
+too_long:
+ Xorriso_msgs_submit(xorriso, 0, "Oversized ACL", 0, "FAILURE", 0);
+ ret= 0;
+ goto ex;
 }
 
 
@@ -863,10 +902,12 @@ int Xorriso_append_extattr_comp(struct XorrisO *xorriso,
                                 char *mode, int flag)
 {
  int ret;
+ size_t line_limit;
  char *line, *wpt, *bsl = NULL;
  unsigned char *upt, *uval;
 
  line= xorriso->result_line;
+ line_limit= sizeof(xorriso->result_line);
  uval= (unsigned char *) comp;
 
  if(*mode == 'q') {
@@ -881,6 +922,9 @@ int Xorriso_append_extattr_comp(struct XorrisO *xorriso,
        strcat(line, "\"$(echo -e '");
      wpt= line + strlen(line);
      for(upt= uval; (size_t) (upt - uval) < comp_len; upt++) {
+       if(wpt - line + 5 + 3 + 1 > (ssize_t) line_limit)
+                                                   /* "\\0xyz" + "')\"" + 0 */
+         goto too_much;
        if(*upt <= 037 || *upt >= 0177 || *upt == '\\' || *upt == '\'') {
          if(flag & 1)
            *(wpt++)= '\\';
@@ -900,10 +944,14 @@ int Xorriso_append_extattr_comp(struct XorrisO *xorriso,
    ret= Sfile_bsl_encoder(&bsl, comp, comp_len, 8);
    if(ret <= 0) 
      {ret= -1; goto ex;}
+   if(strlen(line) + strlen(bsl) + 1 > line_limit)
+     goto too_much;
    strcat(line, bsl);
    free(bsl);
    bsl= NULL;
  } else if(*mode == 'r') {
+   if(strlen(line) + strlen(comp) + 1 > line_limit)
+     goto too_much;
    strcat(line, comp);
  }
  ret= 1;
@@ -911,6 +959,11 @@ ex:;
  if(bsl != NULL)
    free(bsl);
  return(ret);
+too_much:;
+ Xorriso_msgs_submit(xorriso, 0, "Oversized BSD-style file attribute",
+                     0, "FAILURE", 0);
+ ret= -1;
+ goto ex;
 }
 
 
@@ -981,15 +1034,20 @@ int Xorriso_list_extattr(struct XorrisO *xorriso, void *in_node, char *path,
    break;
    if((size_t) (upt - uval) < value_lengths[i]) { 
      strcpy(line, "echo 'OMITTED: Value contains 0-bytes : space \"'\"");
-     Xorriso_append_extattr_comp(xorriso, space_pt, strlen(space_pt), "e", 1);
+     ret= Xorriso_append_extattr_comp(xorriso, space_pt, strlen(space_pt),
+                                      "e", 1);
      if(ret <= 0)
        goto ex;
      strcat(line, "\"'\" , name \"'\"");
-     Xorriso_append_extattr_comp(xorriso, name_pt, strlen(name_pt), "e", 1);
+     ret= Xorriso_append_extattr_comp(xorriso, name_pt, strlen(name_pt),
+                                      "e", 1);
      if(ret <= 0)
        goto ex;
      strcat(line, "\"'\" in file '\"");
-     Xorriso_append_extattr_comp(xorriso, path_pt, strlen(path_pt), "e", 1);
+     ret= Xorriso_append_extattr_comp(xorriso, path_pt, strlen(path_pt),
+                                      "e", 1);
+     if(ret <= 0)
+       goto ex;
      strcat(line, "\" >&2\n");
 
      /* temporarily disable -backslash_codes with result output */
@@ -1340,13 +1398,9 @@ int Xorriso_show_du_subs(struct XorrisO *xorriso, IsoDir *dir_node,
  off_t sub_size, report_size, mem= 0;
  char *path= NULL, *show_path= NULL, *sfe= NULL;
 
- sfe= malloc(5*SfileadrL);
- path= malloc(SfileadrL);
- show_path= malloc(SfileadrL);
- if(path==NULL || show_path==NULL || sfe==NULL) {
-   Xorriso_no_malloc_memory(xorriso, &sfe, 0);
-   {ret= -1; goto ex;}
- }
+ Xorriso_alloc_meM(sfe, char, 5 * SfileadrL);
+ Xorriso_alloc_meM(path, char, SfileadrL);
+ Xorriso_alloc_meM(show_path, char, SfileadrL);
 
  *size= 0;
  ret= iso_dir_get_children(dir_node, &iter);
@@ -1458,12 +1512,9 @@ much_too_long:;
  }
  ret= 1;
 ex:;
- if(sfe!=NULL)
-   free(sfe);
- if(path!=NULL)
-   free(path);
- if(show_path!=NULL)
-   free(show_path);
+ Xorriso_free_meM(sfe);
+ Xorriso_free_meM(path);
+ Xorriso_free_meM(show_path);
  if(iter!=NULL)
    iso_dir_iter_free(iter);
  if(node_array!=NULL)
@@ -1557,10 +1608,91 @@ int Xorriso_node_eff_hidden(struct XorrisO *xorriso, IsoNode *node, int flag)
 }
 
 
+/* @param flag bit0= do not truncate components which contain any of "*?["
+*/
+int Xorriso_truncate_path_comps(struct XorrisO *xorriso, char *path,
+                                char *buffer, char **resultpt, int flag)
+{
+ char *rpt, *basrpt, *wpt, *baswpt, *cpt;
+ int ended, ret, skip;
+
+ *resultpt= path;
+
+ /* Check component lengths */
+ rpt= path;
+ if(*rpt == '/')
+   rpt++;
+ for(ended= 0; !ended;) {
+   basrpt= rpt;
+   rpt= strchr(basrpt, '/');
+   if(rpt == NULL) {
+     rpt= basrpt + strlen(basrpt);
+     ended= 1;
+   }
+   skip= 0;
+   if(flag & 1) {
+     for(cpt= basrpt; cpt < rpt; cpt++) {
+       if(strchr("*?[", *cpt) != NULL) {
+         skip= 1;
+     break;
+       }
+     }
+   }
+   if((!skip) && rpt - basrpt > xorriso->file_name_limit) {
+     ended= 0;
+ break;
+   }
+   if(*rpt == '/')
+     rpt++;
+ }    
+ if(ended)
+   return(1); /* All short enough */
+
+ /* Some truncation is needed */ 
+ buffer[0]= 0;
+ wpt= buffer;
+ if(path[0] == '/')
+   *(wpt++)= '/';
+ rpt= path;
+ if(*rpt == '/')
+   rpt++;
+ for(ended= 0; !ended;) {
+   basrpt= rpt;
+   baswpt= wpt;
+   rpt= strchr(basrpt, '/');
+   if(rpt == NULL) {
+     rpt= basrpt + strlen(basrpt);
+     ended= 1;
+   }
+   skip= 0;
+   for(cpt= basrpt; cpt < rpt; cpt++) {
+     *(wpt++)= *cpt;
+     if((flag & 1) && strchr("*?[", *cpt) != NULL)
+       skip= 1;
+   }
+   *wpt= 0;
+   if((!skip) && rpt - basrpt > xorriso->file_name_limit) {
+      ret= iso_truncate_leaf_name(1, xorriso->file_name_limit, baswpt, 0);
+      Xorriso_process_msg_queues(xorriso, 0);
+      if(ret < 0)
+        return(0);
+      wpt= baswpt + strlen(baswpt);
+   }
+   if(!ended) {
+     *(wpt++)= '/';
+     rpt++;
+   }
+ }
+ *resultpt= buffer;
+ return(1);
+}
+
+
 /* @param flag bit0= long format
                bit1= do not print count of nodes
                bit2= du format
                bit3= print directories as themselves (ls -d)
+               bit5= do not look for truncated versions of path component
 */
 int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
                      int filec, char **filev, off_t boss_mem, int flag)
@@ -1569,7 +1701,7 @@ int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
  IsoNode *node;
  IsoImage *volume;
  char *path= NULL, *link_target= NULL, *rpt, **dfilev= NULL;
- char *a_text= NULL, *d_text= NULL;
+ char *a_text= NULL, *d_text= NULL, *namept;
  off_t size;
  struct stat stbuf;
 
@@ -1591,7 +1723,8 @@ int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
      was_error++;
  continue;
    }
-   ret= Xorriso_node_from_path(xorriso, volume, path, &node, 1);
+   ret= Xorriso_node_from_path(xorriso, volume, path, &node,
+                               1 | ((flag >> 4) & 2));
    if(ret<=0) {
      sprintf(xorriso->info_text, "Not found in ISO image: ");
      Text_shellsafe(path, xorriso->info_text, 1);
@@ -1640,7 +1773,7 @@ int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
          Xorriso_result(xorriso,0);
        }
        Xorriso_ls_filev(xorriso, path,
-                        dfilec, dfilev, boss_mem, (flag&1)|2|8);
+                        dfilec, dfilev, boss_mem, (flag & 1) | 2 | 8);
      }
      if(dfilec>0)
        Sfile_destroy_argv(&dfilec, &dfilev, 0);
@@ -1678,12 +1811,13 @@ int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
      else
        sprintf(rpt, "%7.f ",(double) (size/1024));
    }
+   Xorriso_truncate_path_comps(xorriso, filev[i], path, &namept, 0);
    if(link_target[0] && (flag&5)==1) {
-     Xorriso_esc_filepath(xorriso, filev[i], xorriso->result_line, 1),
+     Xorriso_esc_filepath(xorriso, namept, xorriso->result_line, 1);
      strcat(xorriso->result_line, " -> ");
      Xorriso_esc_filepath(xorriso, link_target, xorriso->result_line, 1 | 2);
    } else {
-     Xorriso_esc_filepath(xorriso, filev[i], xorriso->result_line, 1);
+     Xorriso_esc_filepath(xorriso, namept, xorriso->result_line, 1);
    }
    strcat(xorriso->result_line, "\n");
    Xorriso_result(xorriso, 0);
@@ -1969,7 +2103,7 @@ int Xorriso_expand_pattern(struct XorrisO *xorriso,
  if(ret<=0)
    goto ex;
  count+= extra_filec;
- mem+= extra_filec*sizeof(char *);
+ (*mem)+= extra_filec * sizeof(char *);
  if(count<=0)
    {ret= 0; goto ex;}
  ret= Xorriso_alloc_pattern_mem(xorriso, *mem, count, filev, !!(flag & 4));
@@ -2001,8 +2135,7 @@ int Xorriso_expand_pattern(struct XorrisO *xorriso,
  ret= 1;
 ex:;
  if(ret<=0) {
-   if(filev!=NULL)
-     Sfile_destroy_argv(&count, filev, 0);
+   Sfile_destroy_argv(&count, filev, 0);
    *filec= 0;
  }
  return(ret);
@@ -2011,13 +2144,14 @@ ex:;
 
 int Xorriso__start_end_lbas(IsoNode *node,
                             int *lba_count, int **start_lbas, int **end_lbas,
-                            off_t *size, int flag)
+                            off_t **section_sizes, off_t *size, int flag)
 {
  int section_count= 0, ret, i;
  struct iso_file_section *sections= NULL;
 
  *lba_count= 0;
  *start_lbas= *end_lbas= NULL;
+ *section_sizes= NULL;
  *size= 0;
  if(!LIBISO_ISREG(node))
    return(0);
@@ -2030,13 +2164,15 @@ int Xorriso__start_end_lbas(IsoNode *node,
    {ret= 0; goto ex;}
  *start_lbas= calloc(section_count, sizeof(int));
  *end_lbas= calloc(section_count, sizeof(int));
- if(*start_lbas == NULL || *end_lbas == NULL)
+ *section_sizes= calloc(section_count, sizeof(off_t));
+ if(*start_lbas == NULL || *end_lbas == NULL || *section_sizes == NULL)
    {ret= -1; goto ex;}
  for(i= 0; i < section_count; i++) {
    (*start_lbas)[i]= sections[i].block;
    (*end_lbas)[i]= sections[i].block + sections[i].size / 2048 - 1;
    if(sections[i].size % 2048)
      (*end_lbas)[i]++;
+   (*section_sizes)[i]= sections[i].size;
  }
  *lba_count= section_count;
  ret= 1;
@@ -2059,11 +2195,11 @@ int Xorriso__file_start_lba(IsoNode *node,
                            int *lba, int flag)
 {
  int *start_lbas= NULL, *end_lbas= NULL, lba_count= 0, i, ret;
- off_t size;
+ off_t size, *section_sizes= NULL;
 
  *lba= -1;
  ret= Xorriso__start_end_lbas(node, &lba_count, &start_lbas, &end_lbas,
-                              &size, 0);
+                              &section_sizes, &size, 0);
  if(ret <= 0)
    return(ret);
  for(i= 0; i < lba_count; i++) {
@@ -2074,6 +2210,8 @@ int Xorriso__file_start_lba(IsoNode *node,
    free((char *) start_lbas);
  if(end_lbas != NULL)
    free((char *) end_lbas);
+ if(section_sizes != NULL)
+   free((char *) section_sizes);
  if(*lba < 0)
    return(0);
  return(1);
@@ -2186,6 +2324,8 @@ int Xorriso_retrieve_disk_path(struct XorrisO *xorriso, IsoNode *node,
 }
 
 
+/* @param flag bit0= show numbers from iso_stream_get_id
+ */
 int Xorriso_show_stream(struct XorrisO *xorriso, void *in_node,
                         char *path, int flag)
 {
@@ -2195,6 +2335,9 @@ int Xorriso_show_stream(struct XorrisO *xorriso, void *in_node,
  IsoStream *stream= NULL, *input_stream;
  IsoExternalFilterCommand *cmd;
  char type_text[16], *source_path= NULL;
+ unsigned int fs_id;
+ dev_t dev_id;
+ ino_t ino_id;
 
  node= (IsoNode *) in_node;
  if(node == NULL) {
@@ -2215,6 +2358,12 @@ int Xorriso_show_stream(struct XorrisO *xorriso, void *in_node,
    strcat(xorriso->result_line, " < ");
    Xorriso_stream_type(xorriso, node, stream, type_text, 0);
    strcat(xorriso->result_line, type_text);
+   if(flag & 1) {
+     iso_stream_get_id(stream, &fs_id, &dev_id, &ino_id);
+     sprintf(xorriso->result_line + strlen(xorriso->result_line),
+             "[%u,%lu,%lu]", fs_id, (unsigned long) dev_id,
+                             (unsigned long) ino_id);
+   }
    ret= iso_stream_get_external_filter(stream, &cmd, 0);
    if(ret < 0) {
      Xorriso_process_msg_queues(xorriso,0);
@@ -2234,6 +2383,11 @@ int Xorriso_show_stream(struct XorrisO *xorriso, void *in_node,
  strcat(xorriso->result_line, " < ");
  Xorriso_stream_type(xorriso, node, stream, type_text, 0);
  strcat(xorriso->result_line, type_text);
+ if(flag & 1) {
+   iso_stream_get_id(stream, &fs_id, &dev_id, &ino_id);
+   sprintf(xorriso->result_line + strlen(xorriso->result_line), "[%u,%lu,%lu]",
+           fs_id, (unsigned long) dev_id, (unsigned long) ino_id);
+ }
 
  source_path= iso_stream_get_source_path(stream, 0);
  if(source_path != NULL) {
@@ -2261,7 +2415,7 @@ int Xorriso_file_eval_damage(struct XorrisO *xorriso, IsoNode *node,
 {
  int *start_lbas= NULL, *end_lbas= NULL, lba_count= 0, sect;
  int i, sectors, sector_size, ret;
- off_t sect_base= 0, size= 0, byte;
+ off_t sect_base= 0, size= 0, byte, *section_sizes= NULL;
  struct SectorbitmaP *map;
 
  *damage_start= *damage_end= -1;
@@ -2272,7 +2426,7 @@ int Xorriso_file_eval_damage(struct XorrisO *xorriso, IsoNode *node,
  sector_size/= 2048;
 
  ret= Xorriso__start_end_lbas(node, &lba_count, &start_lbas, &end_lbas,
-                              &size, 0);
+                              &section_sizes, &size, 0);
  if(ret <= 0) {
    Xorriso_process_msg_queues(xorriso, 0);
    return(ret);
@@ -2296,20 +2450,24 @@ int Xorriso_file_eval_damage(struct XorrisO *xorriso, IsoNode *node,
    free((char *) start_lbas);
  if(end_lbas != NULL)
    free((char *) end_lbas);
+ if(section_sizes != NULL)
+   free((char *) section_sizes);
  if(*damage_start < 0)
    return(0);
  return(1);
 }
 
 
+/* @param flag  bit0= report_sections : section size rather than total size
+*/
 int Xorriso_report_lba(struct XorrisO *xorriso, char *show_path,
                        IsoNode *node, int flag)
 {
  int ret, *start_lbas= NULL, *end_lbas= NULL, lba_count, i;
- off_t size;
+ off_t size, *section_sizes= NULL;
 
  ret= Xorriso__start_end_lbas(node, &lba_count, &start_lbas, &end_lbas,
-                              &size, 0);
+                              &section_sizes, &size, 0);
  if(ret < 0) {
    Xorriso_process_msg_queues(xorriso, 0);
    {ret= -1; goto ex;}
@@ -2317,6 +2475,8 @@ int Xorriso_report_lba(struct XorrisO *xorriso, char *show_path,
  if(ret == 0)
    {ret= 1; goto ex;} /* it is ok to ignore other types */
  for(i= 0; i < lba_count; i++) {
+   if(flag & 1)
+     size= section_sizes[i];
    sprintf(xorriso->result_line,
            "File data lba: %2d , %8d , %8d , %8.f , ",
            i, start_lbas[i], end_lbas[i] + 1 - start_lbas[i], (double) size);
@@ -2330,6 +2490,8 @@ ex:;
    free((char *) start_lbas);
  if(end_lbas != NULL)
    free((char *) end_lbas);
+ if(section_sizes != NULL)
+   free((char *) section_sizes);
  return(ret);
 }
 
@@ -2411,7 +2573,9 @@ int Xorriso_search_hardlinks(struct XorrisO *xorriso, IsoNode *node,
 }
 
 
-/* @param flag bit0=do not complain about non existent node */
+/* @param flag bit0=do not complain about non existent node
+               bit1= do not try to find truncated name first
+ */
 int Xorriso_node_from_path(struct XorrisO *xorriso, IsoImage *volume,
                            char *path, IsoNode **node, int flag)
 {
@@ -2427,7 +2591,11 @@ int Xorriso_node_from_path(struct XorrisO *xorriso, IsoImage *volume,
      return(ret);
  }
  *node= NULL;
- ret= iso_tree_path_to_node(volume, path_pt, node);
+ ret= 2;
+ if(!(flag & 2))
+   ret= iso_image_path_to_node(volume, path_pt, node);
+ if(ret == 2)
+   ret= iso_tree_path_to_node(volume, path_pt, node);
  Xorriso_process_msg_queues(xorriso,0);
  if(ret<=0 || (*node)==NULL) {
    if(!(flag&1)) {
@@ -2442,6 +2610,9 @@ int Xorriso_node_from_path(struct XorrisO *xorriso, IsoImage *volume,
 }
 
 
+/* @param flag bit0=do not complain about non existent node
+               bit1= do not try to find truncated name first
+ */
 int Xorriso_dir_from_path(struct XorrisO *xorriso, char *purpose,
                           char *path, IsoDir **dir_node, int flag)
 {
@@ -2453,7 +2624,7 @@ int Xorriso_dir_from_path(struct XorrisO *xorriso, char *purpose,
  if(ret<=0)
    return(ret);
 
- ret= Xorriso_node_from_path(xorriso, volume, path, &node, 0);
+ ret= Xorriso_node_from_path(xorriso, volume, path, &node, flag & 3);
  if(ret<=0)
    goto wdi_is_not_a_dir;
  if(LIBISO_ISDIR(node))
