@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -255,6 +255,11 @@ int Xorriso_detach_libraries(struct XorrisO *xorriso, int flag)
    if(xorriso->libs_are_started==0)
      return(0);
    isoburn_finish();
+
+#ifdef Xorriso_with_editlinE
+   Xorriso__shutdown_editline(0);
+#endif
+
  }
  return(1);
 }
@@ -469,20 +474,23 @@ int Xorriso_set_local_charset(struct XorrisO *xorriso, char *name, int flag)
  nl_charset= nl_langinfo(CODESET);
  if(name == NULL)
    name= nl_charset;
+ if(name == NULL)
+   goto cannot;
 
- if(name != NULL) {
-   iconv_ret= iconv_open(nl_charset, name);
-   if(iconv_ret == (iconv_t) -1)
-     goto cannot;
-   else
-     iconv_close(iconv_ret);
- }
+ iconv_ret= iconv_open(nl_charset, name);
+ if(iconv_ret == (iconv_t) -1)
+   goto cannot;
+ else
+   iconv_close(iconv_ret);
  ret= iso_set_local_charset(name, 0);
  if(ret <= 0) {
 cannot:;
    sprintf(xorriso->info_text,
            "-local_charset: Cannot assume as local character set: ");
-   Text_shellsafe(name, xorriso->info_text, 1);
+   if(name != NULL)
+     Text_shellsafe(name, xorriso->info_text, 1);
+   else
+     Text_shellsafe("(NULL-pointer)", xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
    return(0);
  }
@@ -534,8 +542,8 @@ int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag)
    while(1) {
      tunneled= 0;
      if(pass==0) {
-       ret= 0;
 #ifdef Xorriso_with_libjtE
+       ret= 0;
        if(xorriso->libjte_handle != NULL) {
          msg= libjte_get_next_message(xorriso->libjte_handle);
          if(msg != NULL) {
@@ -547,7 +555,9 @@ int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag)
            ret= 1;
          }
        }
-#endif /* Xorriso_with_libjtE */
+#else
+   break;
+#endif /* ! Xorriso_with_libjtE */
 
      } else if(pass==1)
        ret= iso_obtain_msgs("ALL", &error_code, &imgid, text, severity);
@@ -571,10 +581,14 @@ int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag)
      else if(error_code==0x51001)
        strcpy(severity, "ERRFILE");
 
+#ifdef Xorriso_with_libjtE
      if(pass == 0)
        name_prefix_code= 0;
      else
        name_prefix_code= pass + tunneled;
+#else
+     name_prefix_code= pass + tunneled;
+#endif /* Xorriso_with_libjtE */
      Xorriso_msgs_submit(xorriso, error_code, text, os_errno, 
                          severity, name_prefix_code << 2);
      count++;
@@ -929,10 +943,14 @@ int Xorriso_list_extras(struct XorrisO *xorriso, char *mode, int flag)
  Xorriso_list_extras_result(xorriso, mode, "dvd_obs", 0);
 
  sprintf(xorriso->result_line, "Readline     : %s\n",
+#ifdef Xorriso_with_editlinE
+         "yes , libedit");
+#else
 #ifdef Xorriso_with_readlinE
          "yes");
 #else
          "no");
+#endif
 #endif
  Xorriso_list_extras_result(xorriso, mode, "use_readline", 0);
 
