@@ -1,11 +1,11 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
-   This file contains the implementation of options -a* to -c* as mentioned
+   This file contains the implementation of commands -a* to -c* as mentioned
    in man page or info file derived from xorriso.texi.
 */
 
@@ -347,6 +347,21 @@ int Xorriso_option_application_id(struct XorrisO *xorriso, char *name,
 }
 
 
+/* Command -application_use */
+int Xorriso_option_application_use(struct XorrisO *xorriso, char *path,
+                                   int flag)
+{
+ if(Sfile_str(xorriso->application_use, path, 0) <= 0) {
+   sprintf(xorriso->info_text,
+           "-application_use: parameter string is much too long (%d)",
+           (int) strlen(path));
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -as */
 /* @param flag bit0=do not report the added item
                bit1=do not reset pacifier, no final pacifier message
@@ -535,10 +550,10 @@ int Xorriso_option_biblio_file(struct XorrisO *xorriso, char *name, int flag)
 /* @param flag bit0= format rather than blank
    @return <=0 error , 1 success, 2 revoked by -reassure 
 */
-int Xorriso_option_blank(struct XorrisO *xorriso, char *mode, int flag)
+int Xorriso_option_blank(struct XorrisO *xorriso, char *in_mode, int flag)
 {
- char *cmd= "-blank";
- int aq_ret, ret, mode_flag= 0, as_needed= 0, idx;
+ char *cmd= "-blank", *mode;
+ int aq_ret, ret, mode_flag= 0, as_needed= 0, idx, do_force= 0;
  off_t size= 0;
 
  if(flag&1)
@@ -562,6 +577,11 @@ int Xorriso_option_blank(struct XorrisO *xorriso, char *mode, int flag)
  if(ret<=0)
    return(2);
 
+ if(strncmp(in_mode, "force:", 6) == 0) {
+   do_force= 1;
+   mode= in_mode + 6;
+ } else
+   mode= in_mode;
  if(strcmp(mode, "as_needed")==0 || mode[0]==0)
    as_needed= 1;
  else if(strcmp(mode, "all")==0 || strcmp(mode, "full")==0)
@@ -610,6 +630,8 @@ unusable_size:;
    if(size <= 0)
      goto unusable_size;
    mode_flag= 3;
+ } else if(strcmp(mode, "without_spare") == 0 && (flag & 1)) {
+   mode_flag= 32;
  } else {
    sprintf(xorriso->info_text,
            "%s: Unknown %s mode '%s'",
@@ -617,15 +639,20 @@ unusable_size:;
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(0);
  }
+ if(do_force) {
+   ret= Xorriso_pretend_full_disc(xorriso, 0);
+   if(ret <= 0)
+     return(ret);
+ }
  if(as_needed)
    ret= Xorriso_blank_as_needed(xorriso, (flag&1)<<2);
  else if(flag&1)
-   ret= Xorriso_format_media(xorriso, size, mode_flag & 0xff83);
+   ret= Xorriso_format_media(xorriso, size, mode_flag & 0xffa7);
  else
    ret= Xorriso_blank_media(xorriso, mode_flag&3);
  if(ret==0)
    return(ret);
- if(ret <= 0) { /* in case of success, above functions will have re-aquired */
+ if(ret <= 0) { /* in case of success, above functions will have re-acquired */
    aq_ret= Xorriso_reaquire_outdev(xorriso, 0); /* actually give up drive */
    if(ret<aq_ret)
      return(ret);
@@ -640,9 +667,11 @@ unusable_size:;
 int Xorriso_option_boot_image(struct XorrisO *xorriso, char *form,
                               char *treatment, int flag)
 {
- int was_ok= 1, ret, isolinux_grub= 0, count;
+ int was_ok= 1, ret, isolinux_grub= 0, count, bin_count, parm_len;
+ int palohdrversion;
  unsigned int u;
- char *formpt, *treatpt, *eff_path= NULL;
+ char *formpt, *treatpt, *eff_path= NULL, *eqpt, parm[20];
+ uint8_t sn[8];
  double num;
 
  Xorriso_alloc_meM(eff_path, char, SfileadrL);
@@ -666,7 +695,7 @@ cannot_keep_or_patch:;
    if(isolinux_grub)
      goto treatment_patch;
    xorriso->keep_boot_image= 1;
-   xorriso->patch_isolinux_image= 0;
+   xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 0;
    xorriso->boot_image_bin_path[0]= 0;
    xorriso->patch_system_area= 0;
 
@@ -675,7 +704,7 @@ treatment_patch:;
    if(xorriso->boot_count > 0)
      goto cannot_keep_or_patch;
    xorriso->keep_boot_image= 0;
-   xorriso->patch_isolinux_image= 1;
+   xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 1;
    xorriso->boot_image_bin_path[0]= 0;
    if(strcmp(formpt, "grub") == 0) {
      xorriso->patch_isolinux_image|= 2;
@@ -685,9 +714,14 @@ treatment_patch:;
    else
      xorriso->patch_system_area= 0;
 
+ } else if(strcmp(treatpt, "replay")==0) {
+   ret= Xorriso_report_system_area(xorriso, "cmd", 2);
+   if(ret <= 0)
+     goto ex;
+
  } else if(strcmp(treatpt, "discard")==0) {
    xorriso->keep_boot_image= 0;
-   xorriso->patch_isolinux_image= 0;
+   xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~0x3ff) | 0;
    xorriso->boot_image_bin_path[0]= 0;
    xorriso->patch_system_area= 0;
    if((xorriso->system_area_options & 0xfc ) == 0)
@@ -765,7 +799,7 @@ treatment_patch:;
      strcat(xorriso->boot_image_cat_path, "boot.cat");
      xorriso->boot_image_load_size= 4 * 512;
      xorriso->keep_boot_image= 0;
-     xorriso->patch_isolinux_image= 1;
+     xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 1;
      strcpy(xorriso->boot_image_bin_form, formpt);
      {ret= 1; goto ex;}
 
@@ -789,7 +823,7 @@ treatment_patch:;
      goto ex;
    xorriso->keep_boot_image= 0;
    if(isolinux_grub) {
-     xorriso->patch_isolinux_image= 1;
+     xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 1;
      if(xorriso->boot_image_bin_path[0])
        xorriso->boot_image_load_size= 4 * 512;
      strcpy(xorriso->boot_image_bin_form, formpt);
@@ -836,12 +870,16 @@ treatment_patch:;
 
  } else if(strcmp(treatpt, "mips_discard") == 0 ||
            strcmp(treatpt, "mipsel_discard") == 0 ||
-           strcmp(treatpt, "sparc_discard") == 0) {
+           strcmp(treatpt, "sparc_discard") == 0 || 
+           strcmp(treatpt, "hppa_discard") == 0 ||
+           strcmp(treatpt, "alpha_discard") == 0) {
    xorriso->system_area_options&= ~0xfc; /* system area type 0 */
    Xorriso_add_mips_boot_file(xorriso, "", 1); /* give up MIPS boot files */
+   Xorriso_set_hppa_boot_parm(xorriso, "", "", 1); /* give up HP-PA files */
+   Xorriso_set_alpha_boot(xorriso, "", 1); /* give up DEC Alpha loader */
 
  } else if(strncmp(treatpt, "sparc_label=", 12) == 0) {
-   sprintf(eff_path, "-boot_image %s sparc_path=", formpt);
+   sprintf(eff_path, "-boot_image %s sparc_label=", formpt);
    ret= Xorriso_coordinate_system_area(xorriso, 3, 0, eff_path, 0);
    if(ret <= 0)
      goto ex;
@@ -849,11 +887,60 @@ treatment_patch:;
            Xorriso_disc_label_sizE - 1);
    xorriso->ascii_disc_label[Xorriso_disc_label_sizE - 1] = 0;
 
+ } else if(strncmp(treatpt, "grub2_sparc_core=", 17) == 0) {
+   ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, treatpt + 17,
+                                     xorriso->grub2_sparc_core, 2);
+   if(ret <= 0)
+     goto ex;
+
+ } else if(strncmp(treatpt, "hppa_", 5) == 0) {
+   sprintf(eff_path, "-boot_image %s %s", formpt, treatpt);
+   palohdrversion= (xorriso->system_area_options >> 2) & 0x3f;
+   if(palohdrversion != 4)
+     palohdrversion= 5;
+   ret= Xorriso_coordinate_system_area(xorriso, palohdrversion, 0, eff_path,
+                                       0);
+   if(ret <= 0)
+     goto ex;
+   eqpt= strchr(treatpt, '=');
+   if(eqpt == NULL) {
+     sprintf(xorriso->info_text,
+             "No equal sign found in -boot_image %s %s", form, treatment);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
+   parm_len= (eqpt - treatpt) - 5;
+   if(parm_len > (int) sizeof(parm) - 1)
+     parm_len= sizeof(parm) - 1;
+   strncpy(parm, treatpt + 5, parm_len);
+   parm[parm_len]= 0;
+   ret= Xorriso_set_hppa_boot_parm(xorriso, eqpt + 1, parm, 0);
+   if(ret <= 0)
+     goto ex;
+
+ } else if(strncmp(treatpt, "alpha_boot=", 11) == 0) {
+   sprintf(eff_path, "-boot_image %s %s", formpt, treatpt);
+   ret= Xorriso_coordinate_system_area(xorriso, 6, 0, eff_path, 0);
+   if(ret <= 0)
+     goto ex;
+   ret = Xorriso_set_alpha_boot(xorriso, treatpt + 11, 0);
+   if(ret <= 0)
+     goto ex;
+
  } else if(strncmp(treatpt, "boot_info_table=", 16)==0) {
    if(strcmp(treatpt + 16, "off") == 0)
-     xorriso->patch_isolinux_image= 0;
+     xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 0;
    else if(strcmp(treatpt + 16, "on") == 0)
-     xorriso->patch_isolinux_image= 1 | (2 * (strcmp(treatpt, "grub") == 0));
+     xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) |
+                                    1 | (2 * (strcmp(treatpt, "grub") == 0));
+   else
+     was_ok= 0;
+
+ } else if(strncmp(treatpt, "grub2_boot_info=", 16)==0) {
+   if(strcmp(treatpt + 16, "off") == 0)
+     xorriso->patch_isolinux_image= xorriso->patch_isolinux_image & ~512;
+   else if(strcmp(treatpt + 16, "on") == 0)
+     xorriso->patch_isolinux_image= xorriso->patch_isolinux_image | 512;
    else
      was_ok= 0;
 
@@ -896,9 +983,14 @@ treatment_patch:;
      if(ret <= 0)
        goto ex;
    }
-   ret= Xorriso_set_system_area_path(xorriso, treatpt + 12, 0);
+   if(strcmp(treatpt + 12, ".") == 0) {
+     ret= Xorriso_set_system_area_path(xorriso, "", 0);
+   } else {
+     ret= Xorriso_set_system_area_path(xorriso, treatpt + 12, 0);
+   }
    if(ret <= 0)
      goto ex;
+   xorriso->system_area_options&= ~0x4000;
 
  } else if(strncmp(treatpt, "partition_table=", 16)==0) {
    if(strcmp(treatpt + 16, "off") == 0) {
@@ -913,6 +1005,26 @@ treatment_patch:;
        goto ex;
    } else
      was_ok= 0;
+ } else if(strncmp(treatpt, "partition_entry=", 16)==0) {
+   if(strcmp(formpt, "isolinux") != 0) {
+     sprintf(xorriso->info_text,
+        "-boot_image %s partition_entry=%s : Wrong type. Need \"isolinux\".",
+             formpt, treatpt + 16);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     {ret= 0; goto ex;}
+   }
+   if(strcmp(treatpt + 16, "gpt_basdat") == 0) {
+     xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x0fc) |
+                                     (1 << 2);
+   } else if(strcmp(treatpt + 16, "gpt_hfsplus") == 0) {
+     xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x0fc) |
+                                     (2 << 2);
+   } else if(strcmp(treatpt + 16, "apm_hfsplus") == 0) {
+     xorriso->patch_isolinux_image = xorriso->patch_isolinux_image | (1 << 8);
+   } else if(strcmp(treatpt + 16, "off") == 0) {
+     xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x1fc);
+   } else
+     was_ok= 0;
 
  } else if(strncmp(treatpt, "partition_offset=", 17)==0) {
    u= 0;
@@ -926,6 +1038,14 @@ treatment_patch:;
    }
    xorriso->partition_offset= u;
 
+ } else if(strncmp(treatpt, "appended_part_as=", 17) == 0) {
+   if(strcmp(treatpt + 17, "gpt") == 0) {
+     xorriso->appended_as_gpt = 1;
+   } else if(strcmp(treatpt + 17, "mbr") == 0) {
+     xorriso->appended_as_gpt = 0;
+   } else
+     was_ok= 0;
+
  } else if(strncmp(treatpt, "partition_hd_cyl=", 17)==0) {
    u= 0;
    sscanf(treatpt + 17, "%u", &u);
@@ -957,6 +1077,9 @@ treatment_patch:;
    else if(strcmp(treatpt + 20, "off") == 0)
      xorriso->system_area_options=
                                (xorriso->system_area_options & ~0x300) | 0x200;
+   else if(strcmp(treatpt + 20, "all") == 0)
+     xorriso->system_area_options=
+                               (xorriso->system_area_options & ~0x300) | 0x300;
    else {
      sprintf(xorriso->info_text,
              "-boot_image %s partition_cyl_align: unknown mode : %s",
@@ -966,6 +1089,7 @@ treatment_patch:;
    }
 
  } else if(strncmp(treatpt, "platform_id=", 12)==0) {
+   u= 256; /* intentionally too large */
    if(strncmp(treatpt + 12, "0x", 2) == 0)
      sscanf(treatpt + 14, "%x", &u);
    else
@@ -983,11 +1107,14 @@ treatment_patch:;
    if(strcmp(treatpt + 10, "none") == 0 ||
       strcmp(treatpt + 10, "no_emulation") == 0) {
      xorriso->boot_image_emul= 0;
+     xorriso->boot_emul_default= 0;
    } else if(strcmp(treatpt + 10, "hard_disk") == 0) {
      xorriso->boot_image_emul= 1;
+     xorriso->boot_emul_default= 0;
    } else if(strcmp(treatpt + 10, "floppy") == 0 ||
              strcmp(treatpt + 10, "diskette") == 0) {
      xorriso->boot_image_emul= 2;
+     xorriso->boot_emul_default= 0;
    } else {
      sprintf(xorriso->info_text,
              "-boot_image %s : Unknown media_type : %s",
@@ -996,6 +1123,71 @@ treatment_patch:;
      {ret= 0; goto ex;}
    }
 
+ } else if(strncmp(treatpt, "hfsplus_serial=", 15) == 0) {
+   ret= Hex_to_bin(treatpt + 15, 8, &bin_count, (unsigned char *) sn, 0);
+   if(ret <= 0 || bin_count != 8) {
+     sprintf(xorriso->info_text,
+             "boot_image %s : Malformed hfsplus_serial : %s",
+             formpt, treatpt + 15);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     Xorriso_msgs_submit(xorriso, 0,
+                         "Expected is a string of 16 hex digits [0-9a-fA-F]",
+                         0, "HINT", 0);
+     ret= 0; goto ex;
+   } else {
+     memcpy(xorriso->hfsp_serial_number, sn, 8);
+   }
+
+ } else if(strncmp(treatpt, "hfsplus_block_size=", 19) == 0) {
+   u= 0;
+   sscanf(treatpt + 19, "%u", &u);
+   if(u != 0 && u!= 512 && u != 2048) {
+     sprintf(xorriso->info_text,
+             "boot_image %s : Malformed hfsplus_block_size : %s",
+             formpt, treatpt + 19);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     Xorriso_msgs_submit(xorriso, 0, "Expected are 0, 512, or 2048",
+                         0, "HINT", 0);
+     ret= 0; goto ex;
+   }
+   xorriso->hfsp_block_size= u;
+
+ } else if(strncmp(treatpt, "apm_block_size=", 15) == 0) {
+   u= 0;
+   sscanf(treatpt + 15, "%u", &u);
+   if(u != 0 && u!= 512 && u != 2048) {
+     sprintf(xorriso->info_text,
+             "boot_image %s : Malformed apm_block_size : %s",
+             formpt, treatpt + 15);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     Xorriso_msgs_submit(xorriso, 0, "Expected are 0, 512, or 2048",
+                         0, "HINT", 0);
+     ret= 0; goto ex;
+   }
+   xorriso->apm_block_size= u;
+
+ } else if(strncmp(treatpt, "efi_boot_part=", 14) == 0) {
+   if(Sfile_str(xorriso->efi_boot_partition, treatpt + 14, 0) <= 0)
+     {ret= -1; goto ex;}
+
+ } else if(strncmp(treatpt, "prep_boot_part=", 15) == 0) {
+   if(Sfile_str(xorriso->prep_partition, treatpt + 15, 0) <= 0)
+     {ret= -1; goto ex;}
+
+ } else if(strncmp(treatpt, "chrp_boot_part=", 15) == 0) {
+   if(strcmp(treatpt + 15, "on") == 0) {
+     xorriso->system_area_options= (xorriso->system_area_options & ~0x3cfc) |
+                                   0x400;
+   } else if(strcmp(treatpt + 15, "off") == 0) {
+     xorriso->system_area_options= xorriso->system_area_options & ~0x3c00;
+   } else {
+     sprintf(xorriso->info_text,
+             "-boot_image %s chrp_boot_part: unknown mode : %s",
+             formpt, treatpt + 15);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+     {ret= 0; goto ex;}
+   }
+
  } else if(strncmp(treatpt, "isohybrid=", 10) == 0 &&
            strcmp(formpt, "isolinux")==0) {
 
@@ -1035,6 +1227,22 @@ treatment_patch:;
 
 #endif /* ! Xorriso_with_isohybriD */
 
+ } else if(strncmp(treatpt, "grub2_mbr=", 10) == 0) {
+   ret= Xorriso_coordinate_system_area(xorriso, 0, (1 << 14), eff_path, 1 | 2);
+   if(ret <= 0)
+     goto ex;
+   if(strcmp(treatpt + 10, ".") == 0) {
+     ret= Xorriso_set_system_area_path(xorriso, "", 0);
+   } else {
+     ret= Xorriso_set_system_area_path(xorriso, treatpt + 10, 0);
+   }
+   if(ret <= 0)
+     goto ex;
+   if(treatpt [10] == 0)
+     xorriso->system_area_options&= ~0x4000;
+   else 
+     xorriso->system_area_options|= 0x4000;
+
  } else
    was_ok= 0;
 
@@ -1076,7 +1284,7 @@ int Xorriso_option_calm_drive(struct XorrisO *xorriso, char *which, int flag)
 /* Option -cd alias -cdi */
 int Xorriso_option_cdi(struct XorrisO *xorriso, char *iso_rr_path, int flag)
 {
- char *path= NULL, *eff_path= NULL;
+ char *path= NULL, *eff_path= NULL, *namept;
  int ret;
 
  Xorriso_alloc_meM(path, char, SfileadrL);
@@ -1091,8 +1299,10 @@ int Xorriso_option_cdi(struct XorrisO *xorriso, char *iso_rr_path, int flag)
  Xorriso_warn_of_wildcards(xorriso, iso_rr_path, 1);
  sprintf(xorriso->info_text,"previous working directory:\n");
  Xorriso_info(xorriso,0);
- Text_shellsafe(xorriso->wdi, xorriso->result_line, 0);
- strcat(xorriso->result_line, "/\n");
+ Xorriso_esc_filepath(xorriso, xorriso->wdi, xorriso->result_line, 0);
+ if(xorriso->sh_style_result == 0 || xorriso->wdi[0] == 0)
+   strcat(xorriso->result_line, "/");
+ strcat(xorriso->result_line, "\n");
  Xorriso_result(xorriso,0);
  if(strcmp(iso_rr_path,"/")==0 || iso_rr_path[0]==0) {
    strcpy(xorriso->wdi,"");
@@ -1123,7 +1333,8 @@ int Xorriso_option_cdi(struct XorrisO *xorriso, char *iso_rr_path, int flag)
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    {ret= 0; goto ex;}
  }
- strcpy(xorriso->wdi, eff_path);
+ Xorriso_truncate_path_comps(xorriso, eff_path, path, &namept, 0);
+ strcpy(xorriso->wdi, namept);
 
  Xorriso_option_pwdi(xorriso, 0);
  ret= 1;
@@ -1152,8 +1363,10 @@ int Xorriso_option_cdx(struct XorrisO *xorriso, char *disk_path, int flag)
  Xorriso_warn_of_wildcards(xorriso, disk_path, 1|2);
  sprintf(xorriso->info_text,"previous working directory on hard disk:\n");
  Xorriso_info(xorriso,0);
- Text_shellsafe(xorriso->wdx, xorriso->result_line, 0);
- strcat(xorriso->result_line, "/\n");
+ Xorriso_esc_filepath(xorriso, xorriso->wdx, xorriso->result_line, 0);
+ if(xorriso->sh_style_result == 0 || xorriso->wdx[0] == 0)
+   strcat(xorriso->result_line, "/");
+ strcat(xorriso->result_line, "\n");
  Xorriso_result(xorriso,0);
  if(strcmp(disk_path,"/")==0) {
    strcpy(xorriso->wdx,"");
@@ -1776,10 +1989,19 @@ int Xorriso_option_clone(struct XorrisO *xorriso, char *origin, char *dest,
 }
 
 
-/* Option -close "on"|"off" */
+/* Option -close "on"|"off"|"as_needed" */
 int Xorriso_option_close(struct XorrisO *xorriso, char *mode, int flag)
 {
- xorriso->do_close= !!strcmp(mode, "off");
+ if(strcmp(mode, "off") == 0) {
+   xorriso->do_close= 0;
+   xorriso->auto_close= 0;
+ } else if(strcmp(mode, "as_needed") == 0) {
+   xorriso->do_close= 0;
+   xorriso->auto_close= 1;
+ } else {
+   xorriso->do_close= 1;
+   xorriso->auto_close= 0;
+ }
  return(1);
 }
 
@@ -1821,7 +2043,7 @@ int Xorriso_option_close_filter_list(struct XorrisO *xorriso, int flag)
 
 
 /* Option -commit */
-/* @param flag bit0= leave indrive and outdrive aquired as they were,
+/* @param flag bit0= leave indrive and outdrive acquired as they were,
                      i.e. do not aquire outdrive as new in-out-drive
                bit1= do not perform eventual -reassure
    @return <=0 error , 1 success, 2 revoked by -reassure , 3 no change pending
@@ -1829,6 +2051,7 @@ int Xorriso_option_close_filter_list(struct XorrisO *xorriso, int flag)
 int Xorriso_option_commit(struct XorrisO *xorriso, int flag)
 {
  int ret;
+ char eternal_problem_status_text_mem[80];
 
  if(!Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text, "-commit: No image modifications pending");
@@ -1842,7 +2065,14 @@ int Xorriso_option_commit(struct XorrisO *xorriso, int flag)
      {ret= 2; goto ex;}
  }
  Xorriso_process_errfile(xorriso, 0, "burn session start", 0, 1);
+ Xorriso_get_problem_status(xorriso, eternal_problem_status_text_mem, 1);
  ret= Xorriso_write_session(xorriso, 0);
+ if(ret == 2) {
+   if(Xorriso__severity_cmp("WARNING", eternal_problem_status_text_mem) > 0)
+     strcpy(eternal_problem_status_text_mem, "WARNING");
+   Xorriso_set_problem_status(xorriso, eternal_problem_status_text_mem, 1);
+   ret= Xorriso_retry_write_session(xorriso, 0);
+ }
  Xorriso_process_errfile(xorriso, 0, "burn session end", 0, 1);
  if(ret<=0)
    goto ex;
@@ -2003,6 +2233,89 @@ int Xorriso_option_compliance(struct XorrisO *xorriso, char *mode,
 }
 
 
+/* Command -concat */
+int Xorriso_option_concat(struct XorrisO *xorriso,
+                          int argc, char **argv, int *idx, int flag)
+{
+ int ret, end_idx, optc= 0, progc= 0, iso_rr_start, prog_end_idx= -1;
+ char **optv= NULL, **progv= NULL, *delimiter_mem= NULL;
+ char *delimiter= NULL;
+
+ /* Must be done before any goto ex; */
+ end_idx= Xorriso_end_idx(xorriso, argc, argv, *idx, 1);
+ iso_rr_start= *idx + 2;
+
+ if(xorriso->allow_restore <= 0) {
+   sprintf(xorriso->info_text,
+          "-concat: image content copies are not enabled by option -osirrox");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   ret= 0; goto ex;
+ }
+ if(end_idx - *idx < 3) {
+   sprintf(xorriso->info_text,
+           "-concat: Not enough arguments. Need mode, target, iso_rr_path.");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   ret= 0; goto ex;
+ }
+ if(strcmp(argv[*idx], "pipe") == 0) {
+   if(end_idx - *idx < 5) {
+     sprintf(xorriso->info_text,
+             "-concat pipe: Not enough arguments. Need delimiter, program path, delimiter, iso_rr_path.");
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
+
+   ret= Xorriso_check_thing_len(xorriso, argv[*idx + 1],
+                                sizeof(xorriso->list_delimiter), "-concat",
+                                "Delimiter", 0);
+   if(ret <= 0)
+     goto ex;
+   Xorriso_alloc_meM(delimiter_mem, char, strlen(xorriso->list_delimiter) + 1);
+   Xorriso_alloc_meM(delimiter, char, strlen(argv[*idx + 1]) + 1);
+   strcpy(delimiter_mem, xorriso->list_delimiter);
+   strcpy(delimiter, argv[*idx + 1]);
+   strcpy(xorriso->list_delimiter, delimiter);
+   ret= Xorriso_opt_args(xorriso, "-concat pipe", argc , argv, *idx + 2,
+                         &prog_end_idx, &progc, &progv, 4 | 128);
+   strcpy(xorriso->list_delimiter, delimiter_mem);
+   if(ret <= 0)
+     goto ex;
+   if(progc <= 0) {
+     sprintf(xorriso->info_text, "-concat pipe: No program path given.");
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
+   iso_rr_start= prog_end_idx + 1;
+ }
+ ret= Xorriso_opt_args(xorriso, "-concat", argc , argv, iso_rr_start, &end_idx,
+                       &optc, &optv, 128);
+ if(ret <= 0)
+   goto ex;
+ if(optc <= 0) {
+   sprintf(xorriso->info_text, "-concat: No iso_rr_paths given.");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+   ret= 0; goto ex;
+ }
+ ret= Xorriso_concat(xorriso, argv[*idx], argv[*idx + 1],
+                     progc, progv, optc, optv, 0);
+ex:;
+ if(progv != NULL) {
+   if(delimiter_mem != NULL && delimiter != NULL)
+     strcpy(xorriso->list_delimiter, delimiter);
+   Xorriso_opt_args(xorriso, "-concat", argc, argv, *idx + 2, &prog_end_idx,
+                    &progc, &progv, 256);
+   if(delimiter_mem != NULL && delimiter != NULL)
+     strcpy(xorriso->list_delimiter, delimiter_mem);
+ }
+ Xorriso_opt_args(xorriso, "-concat", argc, argv, iso_rr_start, &end_idx,
+                  &optc, &optv, 256);
+ Xorriso_free_meM(delimiter);
+ Xorriso_free_meM(delimiter_mem);
+ *idx= end_idx;
+ return(ret); 
+}
+
+
 /* Option -copyright_file */
 int Xorriso_option_copyright_file(struct XorrisO *xorriso, char *name, int flag)
 {
