@@ -1,11 +1,11 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
-   This file contains the implementation of options as mentioned in man page
+   This file contains the implementation of commands as mentioned in man page
    or info file derived from xorriso.texi.
 */
 
@@ -29,6 +29,12 @@
 #include "xorriso_private.h"
 #include "xorrisoburn.h"
 
+#ifdef Xorriso_with_readlinE
+#define Xorriso_with_line_editoR
+#endif
+#ifdef Xorriso_with_editlinE
+#define Xorriso_with_line_editoR
+#endif
 
 /* Command -data_cache_size */
 int Xorriso_option_data_cache_size(struct XorrisO *xorriso, char *num_tiles,
@@ -59,7 +65,7 @@ int Xorriso_option_data_cache_size(struct XorrisO *xorriso, char *num_tiles,
                 bit3= regard overwriteable media as blank
                 bit4= if the drive is a regular disk file: truncate it to
                       the write start address
-                bit5= do not print toc of aquired drive
+                bit5= do not print toc of acquired drive
                 bit6= do not calm down drive after aquiring it
     @return <=0 error , 1 success, 2 revoked by -reassure
 */
@@ -340,6 +346,26 @@ int Xorriso_option_early_stdio_test(struct XorrisO *xorriso, char *mode,
 }
 
 
+/* Command -ecma119_map */
+int Xorriso_option_ecma119_map(struct XorrisO *xorriso, char *mode, int flag)
+{
+ if(strcmp(mode, "unmapped") == 0)
+   xorriso->ecma119_map= 0;
+ else if(strcmp(mode, "stripped") == 0)
+   xorriso->ecma119_map= 1;
+ else if(strcmp(mode, "uppercase") == 0)
+   xorriso->ecma119_map= 2;
+ else if(strcmp(mode, "lowercase") == 0)
+   xorriso->ecma119_map= 3;
+ else {
+   sprintf(xorriso->info_text, "-ecma119_map: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -eject */
 /* @param flag bit0=do not report toc of eventually remaining drives
 */
@@ -622,6 +648,29 @@ int Xorriso_option_extract_cut(struct XorrisO *xorriso, char *iso_rr_path,
 }
 
 
+/* Command -file_name_limit */
+int Xorriso_option_file_name_limit(struct XorrisO *xorriso, char *value,
+                                   int flag)
+{
+ int ret, sub_flag= 0;
+ double num;
+
+ if(value[0] == '+')
+   sub_flag|= 1;
+ num= Scanf_io_size(value + (sub_flag & 1), 0);
+ if(num < 64 || num > 255) {
+   sprintf(xorriso->info_text,
+           "-file_name_limit: Value '%s' out of range [64..255]", value);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   return(0);
+ }
+ if(num == xorriso->file_name_limit)
+   return(1);
+ ret= Xorriso_set_file_name_limit(xorriso, (int) num, sub_flag);
+ return(ret > 0);
+}
+
+
 /* Option -file_size_limit */
 int Xorriso_option_file_size_limit(struct XorrisO *xorriso,
                                    int argc, char **argv, int *idx, int flag)
@@ -660,6 +709,40 @@ ex:;
 }
 
 
+static int Xorriso_determine_name_space(struct XorrisO *xorriso,
+                                        char *space_name, int flag)
+{
+ if(strcmp(space_name, "rockridge") == 0)
+   return(1);
+ else if(strcmp(space_name, "joliet") == 0)
+   return(2);
+ else if(strcmp(space_name, "ecma119") == 0 ||
+         strcmp(space_name, "iso9660") == 0)
+   return(3);
+ else if(strcmp(space_name, "hfsplus") == 0)
+   return(4);
+ sprintf(xorriso->info_text, "-find: Unknown output namespace identifier");
+ return(0);
+}
+
+static int Xorriso_truncate_const_find_name(struct XorrisO *xorriso,
+                                            char *expr, char *buffer,
+                                            char **namept, int flag)
+{
+ int ret;
+
+ *namept= expr;
+ ret= Xorriso_truncate_path_comps(xorriso, expr, buffer, namept, 1);
+ if(ret < 0) {
+   sprintf(xorriso->info_text,
+           "-find[ix]: cannot truncate constant -name to -file_name_limit: ");
+   Text_shellsafe(expr, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1);
+}
+
 /* Option -find alias -findi, and -findx */
 /* @param flag bit0= -findx rather than -findi
                bit1= do not reset pacifier, no final pacifier message
@@ -673,11 +756,11 @@ int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
                        int *idx, int flag)
 {
  int ret, i, end_idx, type= 0, action, deleter= 0, start_lba, count;
- int list_extattr_head= 0, bsl_mem, disk_path;
+ int list_extattr_head= 0, bsl_mem, disk_path, name_space;
  struct FindjoB *job, *first_job= NULL, *new_job;
  char *start_path, *path= NULL, *cpt, *other_path_start= NULL, *cd_pt;
  char *access_acl_text= NULL, *default_acl_text= NULL, *list_extattr_mode;
- char *arg1_pt;
+ char *arg1_pt, *namept;
 
  struct stat dir_stbuf;
  uid_t user= 0;
@@ -703,6 +786,7 @@ int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
  if(!(flag&2))
    xorriso->find_compare_result= 1;
  for(i= *idx+1; i<end_idx; i++) {
+   ret= 1;
    if(strcmp(argv[i], "-name")==0) {
      if(i+1>=end_idx) {
 not_enough_arguments:;
@@ -712,7 +796,10 @@ not_enough_arguments:;
        goto sorry_ex;
      }
      i++;
-     ret= Findjob_set_name_expr(job, argv[i], 0);
+     ret= Xorriso_truncate_const_find_name(xorriso, argv[i], path, &namept, 0);
+     if(ret <= 0)
+       goto ex;
+     ret= Findjob_set_name_expr(job, namept, 0);
      if(ret<=0) {
        sprintf(xorriso->info_text, "-find[ix]: cannot set -name expression ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
@@ -722,7 +809,10 @@ not_enough_arguments:;
      if(i+1>=end_idx)
        goto not_enough_arguments;
      i++;
-     ret= Findjob_set_name_expr(job, argv[i], 1);
+     ret= Xorriso_truncate_const_find_name(xorriso, argv[i], path, &namept, 0);
+     if(ret <= 0)
+       goto ex;
+     ret= Findjob_set_name_expr(job, namept, 1);
      if(ret<=0) {
        sprintf(xorriso->info_text,
                "-find[ix]: cannot set -wholename expression ");
@@ -746,6 +836,8 @@ not_enough_arguments:;
      if(i+2>=end_idx)
        goto not_enough_arguments;
      i+= 2;
+
+     /* >>> if letter suffix: use Scanf_io_size */
      sscanf(argv[i-1], "%d", &start_lba);
      sscanf(argv[i], "%d", &count);
      Findjob_set_lba_range(job, start_lba, count, 0);
@@ -831,43 +923,107 @@ not_enough_arguments:;
        sprintf(xorriso->info_text, "-findi: cannot setup -has_hfs_bless test");
        goto sorry_ex;
      }
+   } else if(strcmp(argv[i], "-bad_outname")==0) {
+     if(i + 1 >= end_idx)
+       goto not_enough_arguments;
+     i+= 1;
+     name_space= Xorriso_determine_name_space(xorriso, argv[i], 0);
+     if(name_space < 0) {
+       ret= 0; goto sorry_ex;
+     }
+     ret= Findjob_set_num_filter(job, 21, name_space, 0, 0);
+     if(ret <= 0) {
+       sprintf(xorriso->info_text, "-findi: cannot setup -bad_outname test");
+       goto sorry_ex;
+     }
    } else if(strcmp(argv[i], "-true") == 0) {
      ret= Findjob_set_false(job, -1, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-false") == 0) {
      ret= Findjob_set_false(job, 1, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-decision") == 0) {
      if(i+1>=end_idx)
        goto not_enough_arguments;
      i++;
-     ret= Findjob_set_decision(job, argv[i], 0);
+     ret= Findjob_set_arg1(job, 11, argv[i], 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-prune") == 0) {
      ret= Findjob_set_prune(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-sub") == 0 || strcmp(argv[i], "(") == 0) {
      ret= Findjob_open_bracket(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-subend") == 0 || strcmp(argv[i], ")") == 0) {
      ret= Findjob_close_bracket(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-not") == 0 || strcmp(argv[i], "!") == 0) {
      ret= Findjob_not(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-and") == 0 || strcmp(argv[i], "-a") == 0) {
      ret= Findjob_and(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-or") == 0 || strcmp(argv[i], "-o") == 0) {
      ret= Findjob_or(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-if") == 0) {
      ret= Findjob_if(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-then") == 0) {
      ret= Findjob_then(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-else") == 0) {
      ret= Findjob_else(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-elseif") == 0) {
      ret= Findjob_elseif(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-endif") == 0) {
      ret= Findjob_endif(job, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-sort_lba") == 0) {
      flag|= 8;
      /* If an operator is open: insert a -true test, else do nothing */
      ret= Findjob_set_false(job, -1, 1);
      if(ret == 2)
        ret= 1;
+   } else if(strcmp(argv[i], "-use_pattern") == 0 ||
+             strcmp(argv[i], "-or_use_pattern") == 0) {
+     if(i + 1 >= end_idx)
+       goto not_enough_arguments;
+     i++;
+     ret= Findjob_set_arg1(job,
+                           22 + (strcmp(argv[i - 1], "-or_use_pattern") == 0),
+                           argv[i], 0);
+     if(ret <= 0)
+       goto ex;
+   } else if(strcmp(argv[i], "-name_limit_blocker") == 0) {
+     if(i + 1 >= end_idx)
+       goto not_enough_arguments;
+     i++;
+     sscanf(argv[i], "%d", &count);
+     if(count < 64 || count > 255) {
+       sprintf(xorriso->info_text,
+               "-findi: wrong length with -name_limit_blocker [64...255]");
+       goto sorry_ex;
+     }
+     ret= Findjob_set_num_filter(job, 24, count, 0, 0);
+     if(ret <= 0)
+       goto ex;
    } else if(strcmp(argv[i], "-exec")==0) {
      if(i+1>=end_idx) {
 not_enough_exec_arguments:;
@@ -1139,6 +1295,19 @@ not_enough_exec_arguments:;
        Findjob_set_action_target(job, 47, argv[i], 0);
      } else if(strcmp(cpt, "get_hfs_bless")==0) {
        Findjob_set_action_target(job, 48, NULL, 0);
+     } else if(strcmp(cpt, "print_outname")==0) {
+       if(i+1>=end_idx)
+         goto not_enough_exec_arguments;
+       i++;
+       name_space= Xorriso_determine_name_space(xorriso, argv[i], 0);
+       if(name_space < 0) {
+         ret= 0; goto sorry_ex;
+       }
+       Findjob_set_action_type(job, 50, name_space, 0);
+     } else if(strcmp(cpt, "report_sections")==0) {
+       Findjob_set_action_target(job, 51, NULL, 0);
+     } else if(strcmp(cpt, "show_stream_id") == 0) {
+       Findjob_set_action_target(job, 52, NULL, 0);
      } else {
        sprintf(xorriso->info_text, "-find -exec: unknown action ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
@@ -1226,17 +1395,19 @@ ex:;
 /* Option -follow */
 int Xorriso_option_follow(struct XorrisO *xorriso, char *mode, int flag)
 {
- int was_fl, was_fm, was_fpr, was_fpt, l;
+ int was_fl, was_fm, was_fpr, was_fpt, was_fc, l;
  double num;
  char *cpt, *npt;
 
  was_fpt= xorriso->do_follow_pattern;
  was_fpr= xorriso->do_follow_param;
  was_fl= xorriso->do_follow_links;
+ was_fc= xorriso->do_follow_concat;
  was_fm= xorriso->do_follow_mount;
  xorriso->do_follow_pattern= 0;
  xorriso->do_follow_param= 0;
  xorriso->do_follow_links= 0;
+ xorriso->do_follow_concat= 0;
  xorriso->do_follow_mount= 0;
  npt= cpt= mode;
  for(cpt= mode; npt!=NULL; cpt= npt+1) {
@@ -1251,16 +1422,19 @@ int Xorriso_option_follow(struct XorrisO *xorriso, char *mode, int flag)
      xorriso->do_follow_pattern= 0;
      xorriso->do_follow_param= 0;
      xorriso->do_follow_links= 0;
+     xorriso->do_follow_concat= 0;
      xorriso->do_follow_mount= 0;
    } else if(strncmp(cpt, "on", l)==0) {
      xorriso->do_follow_pattern= 1;
      xorriso->do_follow_param= 1;
      xorriso->do_follow_links= 1;
+     xorriso->do_follow_concat= 1;
      xorriso->do_follow_mount= 1;
    } else if(strncmp(cpt, "default", l)==0) {
      xorriso->do_follow_pattern= 1;
      xorriso->do_follow_param= 0;
      xorriso->do_follow_links= 0;
+     xorriso->do_follow_concat= 0;
      xorriso->do_follow_mount= 1;
      xorriso->follow_link_limit= 100;
    } else if(strncmp(cpt, "link", l)==0 || strncmp(cpt,"links", l)==0) {
@@ -1271,6 +1445,8 @@ int Xorriso_option_follow(struct XorrisO *xorriso, char *mode, int flag)
      xorriso->do_follow_param= 1;
    } else if(strncmp(cpt, "pattern", l)==0) {
      xorriso->do_follow_pattern= 1;
+   } else if(strncmp(cpt, "concat", l)==0) {
+     xorriso->do_follow_concat= 1;
    } else if(strncmp(cpt, "limit=", 6)==0) {
      sscanf(cpt+6, "%lf", &num);
      if(num<=0 || num>1.0e6) {
@@ -1290,6 +1466,7 @@ sorry_ex:
      xorriso->do_follow_pattern= was_fpt;
      xorriso->do_follow_param= was_fpr;
      xorriso->do_follow_links= was_fl;
+     xorriso->do_follow_concat= was_fc;
      xorriso->do_follow_mount= was_fm;
      return(0);
    }
@@ -1317,7 +1494,7 @@ int Xorriso_option_fs(struct XorrisO *xorriso, char *size, int flag)
 }
 
 
-/* Optionis -getfacl alias -getfacli, -getfacl_r alias -getfacl_ri
+/* Commands -getfacl alias -getfacli, -getfacl_r alias -getfacl_ri
             -getfattr alias getfattri
 */
 /* @param flag bit0= recursive -getfacl_r
@@ -1533,17 +1710,22 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 those lists empty. Defaulty entry in \"risky\" is \"/dev\".",
 "  -grow_blindly \"off\"|predicted_nwa",
 "                 Switch between modifying and blind growing.",
+"  -read_speed number[\"k/s\"|\"m/s\"|\"[x]CD\"|\"[x]DVD\"|\"[x]BD\"]",
+"                 Set the read speed. Default is \"none\" = do not set speed",
+"                 before reading.",
 "  -load \"session\"|\"track\"|\"lba\"|\"sbsector\"|\"volid\"|\"auto\" id",
-"                 Load a particular (outdated) ISO image from a -dev or",
+"                 Load a particular (outdated) ISO session from a -dev or",
 "                 -indev which hosts more than one session.",
 "  -displacement [-]block_address",
 "                 When loading ISO tree or reading data files compensate a",
 "                 displacement versus the start address for which the image",
 "                 was prepared.",
+"  -read_fs \"any\"|\"norock\"|\"nojoliet\"|\"ecma119\"",
+"                 Specify which kind of filesystem tree to load if present.",
 "  -rom_toc_scan \"on\"|\"force\"|\"off\"[:\"emul_on\"|\"emul_off\"]",
 "                [:\"emul_wide\"|\"emul_narrow\"]",
 "                 Enable scanning for ISO sessions on read-only drives/media",
-"                 resp. on overwriteable media with emulated TOC.",
+"                 and on overwriteable media with emulated TOC.",
 "  -calm_drive \"in\"|\"out\"|\"all\"|\"on\"|\"off\"",
 "                 Reduce drive noise until it gets actually used again.",
 "  -assert_volid pattern severity",
@@ -1559,21 +1741,24 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -local_charset name",
 "                 Override system assumption of the local character set name.",
 "  -hardlinks mode[:mode ...]",
-"                 Enable resp. disable recording and restoring of hard links.",
+"                 Enable or disable recording and restoring of hard links.",
 "                 Modes are \"on\", \"off\", \"perform_update\",",
 "                 \"without_update\", \"discard_extract\",",
 "                 \"cheap_sorted_extract\", \"normal_extract\"",
 "  -acl \"on\"|\"off\"",
-"                 Enable resp. disable reading and writing of ACLs.",
+"                 Enable or disable reading and writing of ACLs.",
 "  -xattr \"on\"|\"off\"",
-"                 Enable resp. disable reading and writing of xattr.",
+"                 Enable or disable reading and writing of xattr.",
+"  -md5 \"on\"|\"all\"|\"off\"",
+"                 Enable or disable processing of MD5 checksums.",
 "  -for_backup",
 "                 Shortcut for: -hardlinks on -acl on -xattr on -md5 on",
+"  -ecma119_map \"unmapped\"|\"stripped\"|\"uppercase\"|\"lowercase\"",
+"                 Choose conversion of file names if neither Rock Ridge",
+"                 nor Joliet is present in the loaded ISO session.",
 "  -disk_dev_ino \"on\"|\"ino_only\"|\"off\"",
-"                 Enable resp. disable recording of disk file dev_t and ino_t",
+"                 Enable or disable recording of disk file dev_t and ino_t",
 "                 and their use in file comparison.",
-"  -md5 \"on\"|\"all\"|\"off\"",
-"                 Enable resp. disable processing of MD5 checksums.",
 "  -scdbackup_tag list_path record_name",
 "                 Enable production of scdbackup tag with -md5 on",
 "  -ban_stdio_write",
@@ -1583,7 +1768,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -data_cache_size number_of_tiles blocks_per_tile",
 "                 Adjust size and granularity of the data read cache.",
 "  -blank [\"force:\"]\"fast\"|\"all\"|\"deformat\"|\"deformat_quickest\"",
-"                 Blank medium resp. invalidate ISO image on medium.",
+"                 Blank medium or invalidate ISO image on medium.",
 "                 Prefix \"force:\" overrides medium evaluation.",
 "  -close_damaged \"as_needed\"|\"force\"",
 "                 Close track and session of damaged medium.",
@@ -1632,18 +1817,27 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 to which deep subtrees will get relocated if -compliance",
 "                 is set to \"deep_paths_off\".",
 "  -boot_image \"any\"|\"isolinux\"|\"grub\"",
-"              |\"discard\"|\"keep\"|\"patch\"|\"dir=\"|\"bin_path=\"",
+"              |\"discard\"|\"keep\"|\"patch\"|\"replay\"",
+"              |\"dir=\"|\"bin_path=\"",
 "              |\"cat_path=\"|\"cat_hidden=on|iso_rr|joliet|off\"",
 "              |\"load_size=\"|\"boot_info_table=\"",
-"              |\"grub2_boot_info=\"|\"grub2_mbr=\"",
+"              |\"grub2_boot_info=\"|\"grub2_mbr=\"|\"partition_offset=\"",
+"              |\"partition_hd_cyl=\"|\"partition_sec_hd=\"",
+"              |\"partition_cyl_align=\"",
 "              |\"system_area=\"|\"partition_table=on|off\"",
+"              |\"partition_entry=\"|\"appended_part_as=\"",
 "              |\"chrp_boot_part=on|off=\"|\"prep_boot_part=\"",
 "              |\"efi_boot_part=\"|\"efi_boot_part=--efi-boot-image\"",
 "              |\"mips_path=\"|\"mipsel_path=\"|\"mips_discard\"",
 "              |\"sparc_label=\"|\"grub2_sparc_core=\"|\"sparc_discard\"",
+"              |\"hppa_cmdline=\"|\"hppa_bootloader=\"|\"hppa_kernel_32=\"",
+"              |\"hppa_kernel_64=\"|\"hppa_ramdisk=\"|\"hppa_hdrversion=\"",
+"              |\"hppa_discard\"|\"alpha_boot=\"|\"alpha_discard\"",
 "              |\"hfsplus_serial=\"|\"hfsplus_block_size=\"",
 "              |\"apm_block_size=\"|\"show_status\"",
-"                 Whether to discard or keep an exiting El Torito boot image.",
+"                 Whether to discard or keep an exiting El Torito boot image,",
+"                 or to freshly set up boot equipment. \"replay\" performs",
+"                 the commands proposed by -report_system_area \"cmd\".",
 "                 ISOLINUX can be made bootable by dir=/ or dir=/isolinux",
 "                 or dir=/boot/isolinux. Others, like GRUB, by bin_path=...",
 "                 and cat_path=...",
@@ -1655,8 +1849,10 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 efi_boot_part= is for booting EFI systems from USB stick.",
 "                 mips_path= adds Big Endian MIPS boot files. mipsel_path=",
 "                 sets one Little Endian MIPS boot file. sparc_label=",
-"                 activates SUN Disk Label. All three are mutually exclusive",
-"                 and exclusive to MBR production.",
+"                 activates SUN Disk Label. hppa_* is for HP PA-RISC via PALO.",
+"                 alpha_boot= is for DEC Alpha SRM. MIPS, SUN, HP, and Alpha",
+"                 are mutually exclusive and exclusive to production",
+"                 of MBR and to booting via EFI from USB stick.",
 "  -append_partition partition_number type_code disk_path",
 "                 Append a prepared filesystem image after the end of the",
 "                 ISO image. Caution: Will be overwritten by multi-session.",
@@ -1685,12 +1881,27 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -list_speeds   Show media specific list of write speed descriptors.",
 "",
 "  -list_profiles \"in\"|\"out\"|\"all\"",
-"                 Show list of media types supported by indev resp. outdev.",
+"                 Show list of media types supported by indev and/or outdev.",
 "  -print_size    Print the foreseeable consumption by next -commit.",
 "",
 "  -tell_media_space",
 "                 Print foreseeable available space on output medium",
-"  -pvd_info      Print various id strings of the loaded ISO image."
+"  -pvd_info      Print various id strings of the loaded ISO image.",
+"",
+"  -report_el_torito \"plain\"|\"help\"|\"cmd\"|\"as_mkisofs\"",
+"                 \"plain\" prints information about the El Torito boot catalog",
+"                 and boot images of the loaded ISO image.",
+"                 \"help\" prints an explanation of the output format.",
+"                 \"cmd\" and \"as_mkisofs\" propose commands to reproduce",
+"                 the boot equipment reported by -report_el_torito \"plain\"",
+"                 and -report_system_area \"plain\"",
+"  -report_system_area \"plain\"|\"help\"|\"gpt_crc_of:\"disk_path",
+"                      |\"cmd\"|\"as_mkisofs\"",
+"                 \"plain\" prints information about recognized data",
+"                 \"help\" prints an explanation of the output format.",
+"                 in the System Area of the loaded ISO image: MBR, GPT, ...",
+"                 \"gpt_crc_of:\" prints GPT CRC of file disk_path.",
+"                 For \"cmd\" and \"as_mkisofs\" see -report_el_torito.",
 "",
 "Commands with variable length path list [...] need the list delimiter text",
 "as end mark if they are followed by another command. By default this",
@@ -1708,16 +1919,20 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "iso_rr_path is the Rock Ridge name of a file object in the ISO image.",
 "pathspec is either a disk_path or (if allowed) a pair: iso_rr_path=disk_path",
 "Commands marked by [***] have variable length parameter lists and perform",
-"pattern expansion if enabled by -iso_rr_pattern resp. -disk_pattern.",
+"pattern expansion if enabled by -iso_rr_pattern or -disk_pattern.",
 "",
 "  -pathspecs \"on\"|\"off\"  Allow or disallow pathspecs of form ",
 "                 iso_rr_path=disk_path . Only \"off\" allows eventual",
 "                 -disk_pattern expansion.",
+"  -file_name_limit number",
+"                 Set truncation size for file names [64 ... 255].",
+"  -file_size_limit value [...]",
+"                 Set limit for file content size. One or more numbers to add.",
 "  -add pathspec [...] | disk_path [***]",
 "                 Insert the given files or directory trees from",
 "                 filesystem into the ISO image. Much like mkisofs.",
 "  -add_plainly \"none\"|\"unknown\"|\"dashed\"|\"any\"",
-"                 Whether to add lonely arguments as pathspec resp. disk_path.",
+"                 Whether to add lonely arguments as pathspec or disk_path.",
 "  -path_list disk_path",
 "                 Like -add but read the pathspecs from file disk_path.",
 "  -quoted_path_list disk_path",
@@ -1799,17 +2014,20 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                   -type b|c|d|p|f|l|s|e, -pending_data, -hidden,",
 "                   -lba_range start count, -damaged, -has_acl, -has_xattr,",
 "                   -has_aaip, -has_filter, -has_md5, -has_any_xattr,",
-"                   -has_hfs_crtp, -has_hfs_bless,",
+"                   -has_hfs_crtp, -has_hfs_bless, -bad_outname,",
+"                   -name_limit_blocker",
 "                   -prune, -decision yes|no, -true, -false",
 "                 Operators: -not, -or, -and, -sub, (, -subend, ),",
 "                   -if, -then, -elseif, -else, -endif",
 "                 Action may be one of: echo, chown, chown_r, chgrp, chgrp_r",
 "                   chmod, chmod_r, alter_date, alter_date_r, lsdl, compare,",
-"                   rm, rm_r, compare, update, report_damage, report_lba,",
+"                   rm, rm_r, compare, update, report_damage,",
+"                   report_lba, report_sections,",
 "                   getfacl, setfacl, getfattr, setfattr, get_any_xattr,",
 "                   list_extattr, get_md5, check_md5, make_md5,",
 "                   set_hfs_crtp, get_hfs_crtp, set_hfs_bless, get_hfs_bless,",
-"                   set_filter, show_stream, mkisofs_r, hide, find.",
+"                   set_filter, show_stream, show_stream_id, mkisofs_r,",
+"                   hide, print_outname, estimate_size, find",
 "                 params are their parameters except iso_rr_path.",
 "  -mkdir iso_rr_path [...]",
 "                 Create empty directories if they do not exist yet.",
@@ -1836,7 +2054,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Like -not_list but with line rules as -dialog \"on\".",
 "  -not_mgt \"reset\"|\"on\"|\"off\"|\"param_on\"|\"subtree_on\"|\"ignore_on\"",
 "                 Control effect of exclusion lists.",
-"  -follow \"on\"|\"pattern:param:link:mount:limit=#\"|\"default\"|\"off\"",
+"  -follow \"on\"|\"pattern:param:link:concat:mount:limit=#\"|\"default\"|\"off\"",
 "                 Follow symbolic links and mount points within disk_path.",
 "  -overwrite \"on\"|\"nondir\"|\"off\"",
 "                 Allow or disallow to overwrite existing files in ISO image.",
@@ -1897,26 +2115,30 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Give up any unejected drive afterwards.",
 "  -write_type \"auto\"|\"tao\"|\"sao/dao\"",
 "                 Set write type for CD-R[W], DVD-R[W], DVD+R, BD-R.",
-"  -close \"on\"|\"off\"",
+"  -close \"on\"|\"off\"|\"as_needed\"",
 "                 If \"on\" then mark the written medium as not appendable.",
 "  -padding number[\"k\"|\"m\"]|\"included\"|\"appended\"",
 "                 Append extra bytes to image stream. (Default is 300k)",
 "  -dummy \"on\"|\"off\"",
 "                 If \"on\" simulate burning. Refuse if medium cannot simulate.",
 "  -speed number[\"k/s\"|\"m/s\"|\"[x]CD\"|\"[x]DVD\"|\"[x]BD\"]",
-"                 Set the burn speed. Default is 0 = maximum speed.",
+"                 Set the burn speed. Default is \"max\" = maximum speed.",
 "  -stream_recording \"on\"|\"off\"",
 "                 Try to circumvent slow checkread on DVD-RAM, BD-RE, BD-R.",
+"  -modesty_on_drive [\"on\"|\"off\"|min_percent_number]",
+"                    [:\"min_percent=\"number][:\"max_percent=\"number]",
+"                    [:\"min_usec=\"number][:\"max_usec\"=number]",
+"                    [:\"timeout_sec=\"number]",
+"                 Keep drive buffer hungry to ease concurrent burn run.",
 "  -dvd_obs \"default\"|\"32k\"|\"64k\"",
 "                 Set number of bytes per DVD/BD write operation.",
-"  -stdio_sync \"on\"|\"off\"|number",
+"  -stdio_sync \"on\"|\"off\"|\"end\"|number",
 "                 Set number of bytes after which to force output to stdio",
 "                 pseudo drives. \"on\" is the same as 16m.",
 "  -fs number[\"k\"|\"m\"]",
 "                 Set the size of the fifo buffer. (Default is 4m)",
 "  -eject \"in\"|\"out\"|\"all\"",
-"                 Immediately eject the medium in -indev, resp. -outdev,",
-"                 resp. both.",
+"                 Immediately eject the medium in -indev, -outdev, or both.",
 "",
 "Navigation commands:",
 "",
@@ -1963,9 +2185,9 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 local filesystem which match one of the shell parser",
 "                 patterns.",
 "  -dus pattern [***]  like -du but summing up subdirectories without",
-"                 listing them explicitely.",
+"                 listing them explicitly.",
 "  -dusx pattern [***]  like -dux but summing up subdirectories without",
-"                 listing them explicitely.",
+"                 listing them explicitly.",
 "",
 "  -findx disk_path [-name pattern] [-type t] [-exec action [params]]",
 "                 Like -find but operating on local filesystem. Most -exec",
@@ -2017,6 +2239,13 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Like -cp_rx but trying to restore timestamps and ownership.",
 "  -paste_in iso_rr_path disk_path byte_offset byte_count",
 "                 Copy ISO file content into a byte interval of a disk file.",
+"  -concat \"append\"|\"overwrite\" target iso_rr_path [***]",
+"                 Write content of iso_rr_paths into disk file target. Target",
+"                 may be a disk_path or \"-\" for writing to standard output.",
+"  -concat \"pipe\" lim prog [args [...]] lim iso_rr_path [***]",
+"                 Start prog with given args and write content of iso_rr_paths",
+"                 into its standard input. prog must at least contain one '/'.",
+"                 lim may be any word that is not among prog and args.",
 "  -mount drive entity id path",
 "                 Like -mount_cmd but actually performing that command if",
 "                 not setuid or setgid is active.",
@@ -2073,12 +2302,12 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -page len width  Prompt user after len output lines (0=no prompt).",
 "              width (default 80) can adjust line number computation",
 "              to the output terminal's line width.",
-#ifdef Xorriso_with_readlinE
+#ifdef Xorriso_with_line_editoR
 "  -use_readline \"on\"|\"off\"",
 "              Whether to use libreadline for dialog if available.",
 "  -history text  Copy text into libreadline history. This command",
 "              itself is not copied to the history list.",
-#endif /* Xorriso_with_readlinE */
+#endif /* Xorriso_with_line_editoR */
 "  -sh_style_result \"on\"|\"off\"",
 "              If \"on\" do not wrap file addresses in quotation marks with",
 "                -pwd -pwdx -ls -lsd -lsl -lsdl -lsx -lsdx -lslx -lsdlx",
@@ -2145,7 +2374,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -print_mark text",
 "              Print a text to mark channel.",
 "  -prompt text",
-"              Wait for Enter key resp. for a line of input at stdin.",
+"              Wait for Enter key or for a line of input at stdin.",
 "  -sleep number",
 "              Do nothing during the given number of seconds.",
 "  -errfile_log mode path|channel",
