@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -74,6 +74,44 @@ int Xorriso_option_joliet(struct XorrisO *xorriso, char *mode, int flag)
 }
 
 
+/* Command -launch_frontend */
+int Xorriso_option_launch_frontend(struct XorrisO *xorriso,
+                                   int argc, char **argv, int *idx, int flag)
+{
+ int ret, end_idx;
+
+ end_idx= Xorriso_end_idx(xorriso, argc, argv, *idx, 1);
+
+ if(xorriso->launch_frontend_banned) {
+   sprintf(xorriso->info_text,
+           "-launch_frontend was already executed in this xorriso run");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   ret= 0; goto ex;
+ }
+ xorriso->launch_frontend_banned= 1;
+ if(end_idx <= *idx)
+   {ret= 1; goto ex;}
+ if(argv[*idx][0] == 0)
+   {ret= 1; goto ex;}
+ xorriso->dialog= 2;
+ ret= Xorriso_launch_frontend(xorriso, end_idx - *idx, argv + *idx,
+                              "", "", 0);
+ex:;
+ (*idx)= end_idx;
+ return(ret);
+}
+
+
+/* Option -list_arg_sorting */
+int Xorriso_option_list_arg_sorting(struct XorrisO *xorriso, int flag)
+{
+ int ret;
+
+ ret= Xorriso_cmd_sorting_rank(xorriso, 0, NULL, 0, 1);
+ return(ret);
+}
+
+
 /* Option -list_delimiter */
 int Xorriso_option_list_delimiter(struct XorrisO *xorriso, char *text,
                                   int flag)
@@ -115,6 +153,16 @@ int Xorriso_option_list_delimiter(struct XorrisO *xorriso, char *text,
 }
 
 
+/* Option -list_extras */
+int Xorriso_option_list_extras(struct XorrisO *xorriso, char *mode, int flag)
+{
+ int ret;
+
+ ret= Xorriso_list_extras(xorriso, mode, 0);
+ return(ret);
+}
+
+
 /* Option -list_formats */
 int Xorriso_option_list_formats(struct XorrisO *xorriso, int flag)
 {
@@ -162,6 +210,43 @@ int Xorriso_option_list_profiles(struct XorrisO *xorriso, char *which,
 }
 
 
+/* Command -lns alias -lnsi */
+int Xorriso_option_lnsi(struct XorrisO *xorriso, char *target, char *path,
+                        int flag)
+{
+ int ret;
+ char *eff_path= NULL, *buffer= NULL, *namept;
+
+ Xorriso_alloc_meM(eff_path, char, SfileadrL);
+ Xorriso_alloc_meM(buffer, char, SfileadrL);
+
+ ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, path, eff_path, 1);
+ if(ret < 0)
+   {ret= 0; goto ex;}
+ if(ret > 0) { 
+   sprintf(xorriso->info_text, "-lns: Address already existing: ");
+   Text_shellsafe(eff_path, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   {ret= 0; goto ex;}
+ } 
+ ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, path, eff_path, 2);
+ if(ret < 0)
+   {ret= 0; goto ex;}
+ ret= Xorriso_truncate_path_comps(xorriso, target, buffer, &namept, 0);
+ if(ret < 0)
+   {ret= 0; goto ex;}
+ ret= Xorriso_graft_in(xorriso, NULL, namept, eff_path, (off_t) 0, (off_t) 0,
+                       1024);
+ if(ret <= 0)
+   {ret= 0; goto ex;}
+ ret= 1;
+ex:;
+ Xorriso_free_meM(buffer);
+ Xorriso_free_meM(eff_path);
+ return(ret);
+}
+
+
 /* Option -load session|track|sbsector value */
 /* @param flag bit0= with adr_mode sbsector: adr_value is possibly 16 too high
    @return <=0 error , 1 success, 2 revoked by -reassure
@@ -171,7 +256,7 @@ int Xorriso_option_load(struct XorrisO *xorriso, char *adr_mode,
 {
  int ret;
 
- if(xorriso->volset_change_pending) {
+ if(Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text,
            "-load: Image changes pending. -commit or -rollback first");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
@@ -235,19 +320,27 @@ logfile_wrong_form:;
 /* Options -ls  alias -lsi   and -lsl  alias -lsli
        and -lsd alias -lsdi  and -lsdl alias -lsdli
        and -du  alias -dui   and -dus  alias -dusi
-   @param flag bit0= long format (-lsl , -du)
+   @param flag bit0= long format (-lsl , -du, not -dus, not -ls)
                bit1= do not expand patterns but use literally
-               bit2= du rather than ls
-               bit3= list directories as themselves (ls -d) 
+               bit2= -du rather than -ls
+               bit3= list directories as themselves (-lsd) 
 */
 int Xorriso_option_lsi(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag)
 {
- int ret, end_idx, filec= 0, nump, i;
+ int ret, end_idx, filec= 0, nump, i, star= 1;
  char **filev= NULL, **patterns= NULL;
  off_t mem= 0;
  struct stat stbuf;
 
+ if(flag & 4) {
+   if(!(flag & 1))
+     star= 0;
+ } else {
+   if(flag & 8)
+     star= 0;
+ }
+
  end_idx= Xorriso_end_idx(xorriso, argc, argv, *idx, 1);
  if(xorriso->do_iso_rr_pattern==0)
    flag|= 2;
@@ -277,27 +370,30 @@ no_memory:;
      {ret= -1; goto ex;}
    }
    nump= 1;
-   if(flag&8)
-     patterns[0]= ".";
-   else 
+   if(star)
      patterns[0]= "*";
+   else 
+     patterns[0]= ".";
    flag&= ~2;
  } else {
    patterns= calloc(nump, sizeof(char *));
    if(patterns==NULL)
      goto no_memory;
    for(i= 0; i<nump; i++) {
-     if(argv[i + *idx][0]==0)
-       patterns[i]= "*";
-     else
+     if(argv[i + *idx][0]==0) {
+       if(star)
+         patterns[i]= "*";
+       else
+         patterns[i]= ".";
+     } else
        patterns[i]= argv[i + *idx];
    }
  }
 
- if((flag & 1) && !(xorriso->ino_behavior & 1)) {
+ if((flag & 1) && !(xorriso->ino_behavior & 32)) {
    ret= Xorriso_make_hln_array(xorriso, 0); /* for stbuf.st_nlink */
    if(ret < 0)
-     return(ret);
+     goto ex;
  }
  if(flag&2) {
    ret= Xorriso_ls_filev(xorriso, xorriso->wdi, nump, argv + (*idx), mem,
@@ -406,7 +502,7 @@ ex:;
 /* Option -map , -map_single */
 /* @param flag bit0=do not report the added item
                bit1=do not reset pacifier, no final pacifier message
-               bit5=eventually do not insert directory tree
+               bit5= -map_single: do not insert directory tree
 */
 int Xorriso_option_map(struct XorrisO *xorriso, char *disk_path,
                        char *iso_path, int flag)
@@ -581,7 +677,7 @@ int Xorriso_option_map_l(struct XorrisO *xorriso, int argc, char **argv,
    if(ret>0 && !xorriso->request_to_abort)
  continue; /* regular bottom of loop */
    was_failure= 1;
-   fret= Xorriso_eval_problem_status(xorriso, ret, 2);
+   fret= Xorriso_eval_problem_status(xorriso, ret, 1 | 2);
    if(fret>=0)
  continue;
    goto ex;
@@ -607,7 +703,7 @@ int Xorriso_option_map_l(struct XorrisO *xorriso, int argc, char **argv,
                              xorriso->pacifier_total, "", 1);
  else if(mode==1 || mode==2)
    Xorriso_pacifier_callback(xorriso, "content bytes read",
-                             xorriso->pacifier_count, 0, "", 1);
+                             xorriso->pacifier_count, 0, "", 1 | 8 | 32);
  else if(mode==3)
    Xorriso_pacifier_callback(xorriso, "files restored",xorriso->pacifier_count,
                              xorriso->pacifier_total, "", 1|4);
@@ -708,6 +804,94 @@ ex:;
 }
 
 
+int Xorriso_option_modesty_on_drive(struct XorrisO *xorriso, char *mode,
+                                    int flag)
+{
+ char *npt, *cpt, *ppt;
+ int l, num, set_min;
+
+ npt= cpt= mode;
+ for(; npt!=NULL; cpt= npt+1) {
+   npt= strchr(cpt,':');
+   if(npt==NULL)
+     l= strlen(cpt);
+   else
+     l= npt-cpt;
+   if(l == 0)
+ continue;
+   if(l == 3 && strncmp(cpt, "off", l) == 0) {
+     xorriso->modesty_on_drive= 0;
+   } else if(l == 1 && strncmp(cpt, "0", l) == 0) {
+     xorriso->modesty_on_drive= 0;
+   } else if(l == 2 && strncmp(cpt, "on", l) == 0) {
+     xorriso->modesty_on_drive= 1;
+   } else if(l == 1 && strncmp(cpt, "1", l) == 0) {
+     xorriso->modesty_on_drive= 1;
+   } else if(l == 2 && strncmp(cpt, "-1", l) == 0) {
+     ;
+   } else if(*cpt >= '1' && *cpt <= '9') {
+     ppt= cpt;
+     set_min= 2;
+set_size_percent:;
+     sscanf(ppt, "%d", &num);
+     if(num == -1) {
+       ;
+     } else if(num < 25) {
+bad_percent:;
+       sprintf(xorriso->info_text, "-modesty_on_drive: percentage out of range [25 to 100]");
+       Text_shellsafe(cpt, xorriso->info_text, 1);
+       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+       return(0);
+     } else if(num > 100) {
+       goto bad_percent;
+     }
+     if(set_min == 2) {
+       xorriso->modesty_on_drive= 1;
+     }
+     if(set_min)
+       xorriso->min_buffer_percent= num;
+     else
+       xorriso->max_buffer_percent= num;
+   } else if(l >= 12 && strncmp(cpt, "min_percent=", 12) == 0) {
+     ppt= cpt + 12;
+     set_min= 1;
+     goto set_size_percent;
+   } else if(l >= 12 && strncmp(cpt, "max_percent=", 12) == 0) {
+     ppt= cpt + 12;
+     set_min= 0;
+     goto set_size_percent;
+   } else if(l >= 8 && strncmp(cpt, "min_usec=", 8) == 0) {
+     ppt= cpt + 8;
+     set_min= 1;
+set_sec:;
+     sscanf(ppt, "%d", &num);
+     if(num < 0)
+       num= 0;
+     if(set_min == 1)
+       xorriso->min_buffer_usec= num;
+     else if(set_min == 0)
+       xorriso->max_buffer_percent= num;
+     else
+       xorriso->buffer_timeout_sec= num;
+   } else if(l >= 8 && strncmp(cpt, "max_usec=", 8) == 0) {
+     ppt= cpt + 8;
+     set_min= 0;
+     goto set_sec;
+   } else if(l >= 11 && strncmp(cpt, "timeout_sec=", 11) == 0) {
+     ppt= cpt + 11;
+     set_min= -1;
+     goto set_sec;
+   } else {
+     sprintf(xorriso->info_text, "-modesty_on_drive: unknown mode ");
+     Text_shellsafe(cpt, xorriso->info_text, 1);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+ }
+ return(1);
+}
+
+
 /* Options -mount , -mount_cmd , -session_string */
 /* @param bit0= -mount_cmd: print mount command to result channel rather
                             than performing it
@@ -731,7 +915,7 @@ int Xorriso_option_mount(struct XorrisO *xorriso, char *dev, char *adr_mode,
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
      return(0);
    }
-   if(xorriso->volset_change_pending) {
+   if(Xorriso_change_is_pending(xorriso, 0)) {
      sprintf(xorriso->info_text,
              "%s: Image changes pending. -commit or -rollback first", mnt);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
@@ -786,6 +970,159 @@ unknown_mode:;
 }
 
 
+/* Command -move */
+int Xorriso_option_move(struct XorrisO *xorriso, char *origin, char *dest,
+                        int flag)
+{
+ int ret;
+ char *eff_origin= NULL, *eff_dest= NULL;
+
+ Xorriso_alloc_meM(eff_origin, char, SfileadrL);
+ Xorriso_alloc_meM(eff_dest, char, SfileadrL);
+
+ ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, origin, eff_origin, 0);
+ if(ret <= 0)
+   {ret= 0; goto ex;}
+ ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, dest, eff_dest, 2);
+ if(ret < 0)
+   {ret= 0; goto ex;}
+ ret= Xorriso_rename(xorriso, NULL, eff_origin, eff_dest, 0);
+ if(ret <= 0)
+   goto ex;
+ ret= 1;
+ex:;
+ Xorriso_free_meM(eff_origin);
+ Xorriso_free_meM(eff_dest);
+ return(ret);
+}
+
+ 
+/* Command -msg_op */
+int Xorriso_option_msg_op(struct XorrisO *xorriso, char *what, char *arg,
+                          int flag)
+{
+ int ret, available, argc, pargc, i, pflag, max_words, input_lines, msd_mem;
+ char **argv= NULL, **pargv= NULL, *msg= "";
+ char *prefix, *separators;
+
+ msd_mem= xorriso->msg_sieve_disabled;
+
+ ret= 1;
+ if(strcmp(what, "parse") == 0 || strcmp(what, "parse_silently") == 0 ||
+    strcmp(what, "parse_bulk") == 0 ||
+    strcmp(what, "parse_bulk_silently") == 0) {
+   ret= Xorriso_parse_line(xorriso, arg, "", "", 5, &argc, &argv, 0);
+   prefix= "";
+   if(argc > 0) 
+     prefix= argv[0];
+   separators= "";
+   if(argc > 1)
+     separators= argv[1];
+   max_words= 0;
+   if(argc > 2)
+     sscanf(argv[2], "%d", &max_words);
+   pflag= 0;
+   if(argc > 3)
+     sscanf(argv[3], "%d", &pflag);
+   input_lines= 1;
+   if(argc > 4)
+     sscanf(argv[4], "%d", &input_lines);
+   if(strcmp(what, "parse") == 0 || strcmp(what, "parse_silently") == 0) {
+     ret= Xorriso_msg_op_parse(xorriso, "", prefix, separators,
+                               max_words, pflag, input_lines,
+                               (strcmp(what, "parse_silently") == 0));
+   } else {
+     ret= Xorriso_msg_op_parse_bulk(xorriso, prefix, separators,
+                               max_words, pflag, input_lines,
+                               (strcmp(what, "parse_bulk_silently") == 0));
+   }
+   if(ret <= 0)
+     goto ex;
+   xorriso->msg_sieve_disabled= msd_mem;
+   Xorriso__dispose_words(&argc, &argv);
+ } else if(strcmp(what, "start_sieve") == 0) {
+   Xorriso_sieve_dispose(xorriso, 0);
+   ret= Xorriso_sieve_big(xorriso, 0);
+   msg= "Message sieve enabled";
+
+/* >>> } else if(strcmp(what, "add_filter_rule") == 0) { */;
+
+ } else if(strcmp(what, "clear_sieve") == 0) {
+   ret= Xorriso_sieve_clear_results(xorriso, 0);
+   msg= "Recorded message sieve results disposed";
+ } else if(strcmp(what, "end_sieve") == 0) {
+   ret= Xorriso_sieve_dispose(xorriso, 0);
+   msg= "Message sieve disabled";
+ } else if(strcmp(what, "read_sieve") == 0) {
+   ret= Xorriso_sieve_get_result(xorriso, arg, &pargc, &pargv, &available, 0);
+   xorriso->msg_sieve_disabled= 1;
+   sprintf(xorriso->result_line, "%d\n", ret);
+   Xorriso_result(xorriso, 1);
+   if(ret > 0) {
+     sprintf(xorriso->result_line, "%d\n", pargc);
+     Xorriso_result(xorriso, 1);
+     for(i= 0; i < pargc; i++) {
+       ret= Sfile_count_char(pargv[i], '\n') + 1;
+       sprintf(xorriso->result_line, "%d\n", ret);
+       Xorriso_result(xorriso, 1);
+       Sfile_str(xorriso->result_line, pargv[i], 0);
+       strcat(xorriso->result_line, "\n");
+       Xorriso_result(xorriso, 1);
+     }
+   } else {
+     strcpy(xorriso->result_line, "0\n");
+     Xorriso_result(xorriso, 1);
+     available= 0;
+   }
+   sprintf(xorriso->result_line, "%d\n", available);
+   Xorriso_result(xorriso, 1);
+   xorriso->msg_sieve_disabled= msd_mem;
+   Xorriso__dispose_words(&pargc, &pargv);
+   ret= 1;
+ } else if(strcmp(what, "show_sieve") == 0) {
+   ret= Xorriso_sieve_get_result(xorriso, "", &pargc, &pargv, &available, 8);
+   xorriso->msg_sieve_disabled= 1;
+   sprintf(xorriso->result_line, "%d\n", ret);
+   Xorriso_result(xorriso, 1);
+   if(ret > 0) {
+     sprintf(xorriso->result_line, "%d\n", pargc);
+     Xorriso_result(xorriso, 1);
+     for(i= 0; i < pargc; i++) {
+       sprintf(xorriso->result_line, "%s\n", pargv[i]);
+       Xorriso_result(xorriso, 1);
+     }
+   }
+   xorriso->msg_sieve_disabled= msd_mem;
+   Xorriso__dispose_words(&pargc, &pargv);
+ } else if(strcmp(what, "compare_sev") == 0) {
+   ret= Xorriso_parse_line(xorriso, arg, "", ",", 2, &argc, &argv, 0);
+   if(argc < 2) {
+     sprintf(xorriso->info_text,
+             "-msg_op cmp_sev: malformed severity pair '%s'", arg);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   } else {
+     ret= Xorriso__severity_cmp(argv[0], argv[1]);
+     sprintf(xorriso->result_line, "%d\n", ret);
+     Xorriso_result(xorriso, 1);
+   }
+   Xorriso__dispose_words(&argc, &argv);
+ } else if(strcmp(what, "list_sev") == 0) {
+   sprintf(xorriso->result_line, "%s\n", Xorriso__severity_list(0));
+   Xorriso_result(xorriso, 1);
+ } else {
+   sprintf(xorriso->info_text, "-msg_op: unknown operation '%s'", what);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   ret= 0;
+ }
+ if(ret > 0 && msg[0])
+   Xorriso_msgs_submit(xorriso, 0, msg, 0, "NOTE", 0);
+
+ex:;
+ xorriso->msg_sieve_disabled= msd_mem;
+ return(ret);
+}
+
+
 /* Option -mv alias -mvi */
 int Xorriso_option_mvi(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag)
@@ -857,6 +1194,48 @@ ex:;
 }
 
 
+/* Option -named_pipe_loop */
+int Xorriso_option_named_pipe_loop(struct XorrisO *xorriso, char *mode,
+                                   char *stdin_pipe, char *stdout_pipe,
+                                   char *stderr_pipe, int flag)
+{
+ char *pipe_paths[3], *cpt, *npt;
+ int ret, hflag= 0, l;
+
+ npt= mode;
+ for(cpt= mode; npt != NULL; cpt= npt + 1) {
+   npt= strchr(cpt, ':');
+   if(npt==NULL)
+     l= strlen(cpt);
+   else
+     l= npt-cpt;
+   if(l==0) {
+     ;
+   } else if(strncmp(cpt, "-", l) == 0) {
+     ;
+   } else if(strncmp(cpt, "cleanup", l) == 0) {
+     hflag|= 1;
+   } else if(strncmp(cpt, "keep", l) == 0) {
+     hflag&= ~1;
+   } else if(strncmp(cpt, "buffered", l) == 0) {
+     hflag|= 2;
+   } else if(strncmp(cpt, "direct", l) == 0) {
+     hflag&= ~2;
+   } else { 
+     sprintf(xorriso->info_text, "-named_pipe_loop: unknown mode in '%s'",
+             mode);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+ }
+ pipe_paths[0]= stdin_pipe;
+ pipe_paths[1]= stdout_pipe;
+ pipe_paths[2]= stderr_pipe;
+ ret= Xorriso_named_pipe_loop(xorriso, pipe_paths, hflag);
+ return(ret);
+}
+
+
 /* Option -no_rc */
 int Xorriso_option_no_rc(struct XorrisO *xorriso, int flag)
 {
@@ -865,9 +1244,11 @@ int Xorriso_option_no_rc(struct XorrisO *xorriso, int flag)
 }
 
 
-/* Option -not_leaf , (-hide_disk_leaf resp. -as mkisofs -hide) */
-/* @param flag  bit0= add to iso_rr_hidings rather than disk_exclusions
-                bit1= add to joliet_hidings rather than disk_exclusions
+/* Option -not_leaf , (-hide_disk_leaf , -as mkisofs -hide) */
+/* @param flag  bit0-bit5= hide rather than adding to disk_exclusions
+                  bit0= add to iso_rr_hidings
+                  bit1= add to joliet_hidings
+                  bit2= add to hfsplus_hidings
 */
 int Xorriso_option_not_leaf(struct XorrisO *xorriso, char *pattern, int flag)
 {
@@ -882,7 +1263,7 @@ int Xorriso_option_not_leaf(struct XorrisO *xorriso, char *pattern, int flag)
  Xorriso__bourne_to_reg(pattern, regexpr, 0);
  if(regcomp(&re, regexpr, 0)!=0)
    {ret= 0; goto cannot_add;}
- if(flag & 3) {
+ if(flag & 63) {
    if(flag & 1) {
      ret= Exclusions_add_not_leafs(xorriso->iso_rr_hidings, pattern, &re, 0);
      if(ret<=0)
@@ -893,6 +1274,11 @@ int Xorriso_option_not_leaf(struct XorrisO *xorriso, char *pattern, int flag)
      if(ret<=0)
        goto cannot_add;
    }
+   if(flag & 4) {
+     ret= Exclusions_add_not_leafs(xorriso->hfsplus_hidings, pattern, &re, 0);
+     if(ret<=0)
+       goto cannot_add;
+   }
  } else {
    ret= Exclusions_add_not_leafs(xorriso->disk_exclusions, pattern, &re, 0);
  }
@@ -951,9 +1337,9 @@ int Xorriso_option_not_list(struct XorrisO *xorriso, char *adr, int flag)
  }
  ret= 1;
 ex:;
+ Xorriso_read_lines(xorriso, fp, &linecount, &argc, &argv, 2);
  if(fp != NULL && fp != stdin)
    fclose(fp);
- Xorriso_read_lines(xorriso, fp, &linecount, &argc, &argv, 2);
  if(ret<=0) {
    sprintf(xorriso->info_text, "Aborted reading of file ");
    Text_shellsafe(adr, xorriso->info_text, 1);
@@ -1030,17 +1416,20 @@ ex:;
 }
 
 
-/* Option -not_paths , (-hide_disk_paths resp. -as mkisofs -hide) */
+/* Option -not_paths , (-hide_disk_paths , -as mkisofs -hide) */
 /* @param flag  bit0= add to iso_rr_hidings rather than disk_exclusions
                 bit1= add to joliet_hidings rather than disk_exclusions
                 bit2= enable disk pattern expansion regardless of -disk_pattern
-             
+                bit8-13= consolidated hide state bits, duplicating bit0-1
+                   bit8= add to iso_rr_hidings
+                   bit9= add to joliet_hidings
+                  bit10= add to hfsplus_hidings
 */
 int Xorriso_option_not_paths(struct XorrisO *xorriso, int argc, char **argv,
                              int *idx, int flag)
 {
  int ret, end_idx, num_descr= 0, dummy, optc= 0, i;
- char **descr= NULL, **optv= NULL, *eff_path= NULL;
+ char **descr= NULL, **optv= NULL, *eff_path= NULL, *hpt= NULL;
 
  end_idx= Xorriso_end_idx(xorriso, argc, argv, *idx,
                           (xorriso->do_disk_pattern == 1 || (flag & 4)) | 2);
@@ -1071,20 +1460,23 @@ no_memory:;
      goto no_memory;
  }
 
- ret= Xorriso_opt_args(xorriso, (flag & 3) ? "-hide_disk_paths" : "-not_paths",
+ ret= Xorriso_opt_args(xorriso,
+                       (flag & 0x3f03) ? "-hide_disk_paths" : "-not_paths",
                        num_descr, descr, 0, &dummy, &optc, &optv,
                        2 | ((flag & 4) << 7));
  if(ret<=0)
    goto ex; 
- if(flag & 3) {
-   if(flag & 1) {
+ if(flag & 0x3f03) {
+   if(flag & 0x0101) {
      ret= Exclusions_add_not_paths(xorriso->iso_rr_hidings,
                                    num_descr, descr, optc, optv, 0);
      if(ret<=0) {
 no_hide:;
-       sprintf(xorriso->info_text,
-               "Cannot add path list: -hide_disk_paths %s ",
-               Xorriso__hide_mode_text(flag & 3, 0));
+       sprintf(xorriso->info_text, "Cannot add path list: -hide_disk_paths ");
+       hpt= Xorriso__hide_mode_text(flag & 0x3f03, 0);
+       if(hpt != NULL)
+         sprintf(xorriso->info_text + strlen(xorriso->info_text), "%s ", hpt);
+       Xorriso_free_meM(hpt);
        Text_shellsafe(argv[*idx], xorriso->info_text, 1);
        strcat(xorriso->info_text, num_descr > 1 ? " ... " : " ");
        strcat(xorriso->info_text, xorriso->list_delimiter);
@@ -1092,12 +1484,18 @@ no_hide:;
        goto ex;
      }
    }
-   if(flag & 2) {
+   if(flag & 0x0202) {
      ret= Exclusions_add_not_paths(xorriso->joliet_hidings,
                                    num_descr, descr, optc, optv, 0);
      if(ret<=0)
        goto no_hide;
    }
+   if(flag & 0x0400) {
+     ret= Exclusions_add_not_paths(xorriso->hfsplus_hidings,
+                                   num_descr, descr, optc, optv, 0);
+     if(ret<=0)
+       goto no_hide;
+   }
  } else {
    ret= Exclusions_add_not_paths(xorriso->disk_exclusions,
                                  num_descr, descr, optc, optv, 0);
@@ -1157,6 +1555,9 @@ return:
  ret= Xorriso_afile_fopen(xorriso, adr, "rb", &fp, 0);
  if(ret <= 0)
    return(0);
+ sprintf(xorriso->info_text, "Command file:  ");
+ Text_shellsafe(adr, xorriso->info_text, 1);
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
  while(1) {
    ret= Xorriso_read_lines(xorriso, fp, &linecount, &linec, &linev, 1 | 8);
    if(ret <= 0)
@@ -1211,6 +1612,9 @@ ex:;
    strcat(xorriso->info_text, "\n");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 1);
  }
+ sprintf(xorriso->info_text, "Command file end:  ");
+ Text_shellsafe(adr, xorriso->info_text, 1);
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
  if(ret!=1)
    return(ret);
  return(!was_failure);
@@ -1234,30 +1638,40 @@ int Xorriso_option_osirrox(struct XorrisO *xorriso, char *mode, int flag)
      l= npt-cpt;
    if(l==0 && mode[0]!=0)
      goto unknown_mode;
-   if(strncmp(cpt, "off", l)==0)
+   if(strncmp(cpt, "off", l)==0 && l >= 3)
      allow_restore= 0;
-   else if(strncmp(cpt, "banned", l)==0)
+   else if(strncmp(cpt, "banned", l)==0 && l >= 5)
      allow_restore= -1;
-   else if(strncmp(cpt, "device_files", l)==0)
-     allow_restore= 2;
-   else if(strncmp(cpt, "on", l)==0 || mode[0]==0)
+   else if(strncmp(cpt, "blocked", l)==0 && l >= 7)
+     allow_restore= -2;
+   else if(strncmp(cpt, "unblock", l)==0 && l >= 7) {
+     if(xorriso->allow_restore == -2)
+       xorriso->allow_restore= 0;
      allow_restore= 1;
-   else if(strncmp(cpt, "concat_split_on", l)==0)
+   } else if(strncmp(cpt, "device_files", l)==0 && l >= 12)
+     allow_restore= 2;
+   else if((strncmp(cpt, "on", l)==0 && l >= 2) || mode[0]==0)
+     allow_restore= 1;
+   else if(strncmp(cpt, "concat_split_on", l)==0 && l >= 15)
      xorriso->do_concat_split= 1;
-   else if(strncmp(cpt, "concat_split_off", l)==0)
+   else if(strncmp(cpt, "concat_split_off", l)==0 && l >= 16)
      xorriso->do_concat_split= 0;
-   else if(strncmp(cpt, "auto_chmod_on", l)==0)
+   else if(strncmp(cpt, "auto_chmod_on", l)==0 && l >= 13)
      xorriso->do_auto_chmod= 1;
-   else if(strncmp(cpt, "auto_chmod_off", l)==0)
+   else if(strncmp(cpt, "auto_chmod_off", l)==0 && l >= 14)
      xorriso->do_auto_chmod= 0;
-   else if(strncmp(cpt, "sort_lba_on", l)==0)
+   else if(strncmp(cpt, "sort_lba_on", l)==0 && l >= 11)
      xorriso->do_restore_sort_lba= 1;
-   else if(strncmp(cpt, "sort_lba_off", l)==0)
+   else if(strncmp(cpt, "sort_lba_off", l)==0 && l >= 12)
      xorriso->do_restore_sort_lba= 0;
-   else if(strncmp(cpt, "o_excl_off", l)==0)
-     xorriso->drives_exclusive= 0;
-   else if(strncmp(cpt, "o_excl_on", l)==0)
+   else if(strncmp(cpt, "o_excl_on", l)==0 && l >= 9)
      xorriso->drives_exclusive= 1;
+   else if(strncmp(cpt, "o_excl_off", l)==0 && l >= 10)
+     xorriso->drives_exclusive= 0;
+   else if(strncmp(cpt, "strict_acl_on", l)==0 && l >= 13)
+     xorriso->do_strict_acl|= 1;
+   else if(strncmp(cpt, "strict_acl_off", l)==0 && l >= 14)
+     xorriso->do_strict_acl&= ~1;
    else {
 unknown_mode:;
      sprintf(xorriso->info_text, "-osirrox: unknown mode '%s'", cpt);
@@ -1271,6 +1685,12 @@ unknown_mode:;
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(0);
  }
+ if(allow_restore > 0 && xorriso->allow_restore == -2) {
+   sprintf(xorriso->info_text,
+           "-osirrox: is currently disabled by setting 'blocked'");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
  if(xorriso->allow_restore != -1)
    xorriso->allow_restore= allow_restore;
  sprintf(xorriso->info_text,
