@@ -2,7 +2,7 @@
 /* xorriso - Command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Initial code of this program was derived from program src/askme.c out
    of scdbackup-0.8.8, Copyright 2007 Thomas Schmitt, BSD-License.
@@ -43,7 +43,7 @@
    --------------------------------------------------------------------------
    The following overview is relevant for development but not for usage of
    xorriso. An application programmer should read xorriso.h and man xorriso
-   resp. info xorriso, rather than diving into its source code.
+   (or info xorriso), rather than diving into its source code.
    For examples see the functions main() and check_compatibility() below.
    --------------------------------------------------------------------------
 
@@ -70,7 +70,7 @@
    aux_objects.c    various helper classes
    misc_funct.c     miscellaneous helper functions
    findjob.c        performs tree searches in libisofs or in POSIX filesystem
-   check_media.c    perform verifying runs on media resp. images
+   check_media.c    perform verifying runs on media or images
    text_io.c        text i/o functions
    match.c          functions for pattern matching
    emulators.c      emulators for mkisofs and cdrecord
@@ -110,7 +110,7 @@
    version of xorriso.
 */
 #define Xorriso_req_majoR  1
-#define Xorriso_req_minoR  3
+#define Xorriso_req_minoR  4
 #define Xorriso_req_micrO  2
 
 
