@@ -4,7 +4,7 @@
    a command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -18,7 +18,7 @@
 /* The minimum version of libisoburn to be used with this version of xorriso
 */
 #define xorriso_libisoburn_req_major  1
-#define xorriso_libisoburn_req_minor  3
+#define xorriso_libisoburn_req_minor  4
 #define xorriso_libisoburn_req_micro  2
 
 
@@ -45,6 +45,8 @@ int Xorriso_give_up_drive(struct XorrisO *xorriso, int flag);
 
 int Xorriso_write_session(struct XorrisO *xorriso, int flag);
 
+int Xorriso_retry_write_session(struct XorrisO *xorriso, int flag);
+
 /* @param boss_iter Opaque handle to be forwarded to actions in ISO image
                     Set to NULL if calling this function from outside ISO world
    @param flag bit0= mkdir: graft in as empty directory, not as copy from disk
@@ -233,13 +235,17 @@ int Xorriso_atip(struct XorrisO *xorriso, int flag);
 int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
                        char *track_source, off_t tsize, int flag);
 
+int Xorriso_retry_burn_track(struct XorrisO *xorriso,  
+                             off_t write_start_address,
+                             char *track_source, off_t tsize, int flag);
+
 /* @param flag bit1= outdev rather than indev
    @return <=0 = failure , 1= ok , 2= ok, is CD profile
 */ 
 int Xorriso_get_profile(struct XorrisO *xorriso, int *profile_number,
                         char profile_name[80], int flag);
 
-/* @param flag bit0= node_pt is a valid ISO object handle, ignore pathname
+/* @param flag bit0= open IsoNode *node_pt rather than looking up pathname
                bit1= dig out the most original stream for reading
 */
 int Xorriso_iso_file_open(struct XorrisO *xorriso, char *pathname,
@@ -250,6 +256,9 @@ int Xorriso_iso_file_read(struct XorrisO *xorriso, void *stream, char *buf,
 
 int Xorriso_iso_file_close(struct XorrisO *xorriso, void **stream, int flag);
 
+int Xorriso_iso_file_to_fd(struct XorrisO *xorriso, char *path, int fd,
+                           int flag);
+
 /* @param bit0= copy link target properties rather than link properties
 */
 int Xorriso_copy_properties(struct XorrisO *xorriso,
@@ -534,6 +543,11 @@ int Xorriso_boot_image_status(struct XorrisO *xorriso, char *filter, FILE *fp,
 
 int Xorriso_add_mips_boot_file(struct XorrisO *xorriso, char *path, int flag);
 
+int Xorriso_set_hppa_boot_parm(struct XorrisO *xorriso, char *text, char *what,
+                               int flag);
+
+int Xorriso_set_alpha_boot(struct XorrisO *xorriso, char *path, int flag);
+
 int Xorriso_coordinate_system_area(struct XorrisO *xorriso, int sa_type,
                                    int options, char *cmd, int flag);
 
@@ -611,6 +625,15 @@ int Xorriso_pretend_full_disc(struct XorrisO *xorriso, int flag);
 
 int Xorriso_apply_sort_file(struct XorrisO *xorriso, char *path, int flag);
 
+int Xorriso_report_system_area(struct XorrisO *xorriso, char *form, int flag);
+
+int Xorriso_external_filter_banned(struct XorrisO *xorriso, char *purpose,
+                                   int flag);
+
+int Xorriso_set_file_name_limit(struct XorrisO *xorriso, int value, int flag);
+
+int Xorriso_truncate_path_comps(struct XorrisO *xorriso, char *path,
+                                char *buffer, char **resultpt, int flag);
 
 #endif /* Xorrisoburn_includeD */
 
